from operator import mul

import numpy as np

import mang.cudamat as cm
from mang.cudamat import cudamat_conv as cm_conv
from mang.edge.edge import Edge


class MaxPoolingEdge(Edge):

    _name = "max_pool"

    def __init__(self, nodes, conn, option={}):
        self.conn = conn
        (node1, node2) = (nodes[conn[0]], nodes[conn[1]])

        self.ratio = option["ratio"]
        self.stride = option["stride"] if "stride" in option else self.ratio
        self.scale = option["scale"] if "scale" in option else 1.

        size_in = reduce(mul, node1.shape)
        size_out = reduce(mul, node2.shape)
        self.shape = (size_in, size_out)
        self.n_channel = 1 if len(node1.shape) == 2 else node1.shape[-1]
        assert self.n_channel == node2.shape[-1]

        self.on_gpu = False
        self.W = np.ones((1, 1))  # dummy weights for consistent interface
        self.o = None
        self.dy = None
        self.used_gpu_memory = 0

    def to_gpu(self, batch_size):
        if not self.on_gpu:
            self.o = cm.empty((batch_size, self.shape[1]))
            self.used_gpu_memory += self.o.shape[0] * self.o.shape[1] * 4
            self.dy = cm.empty((batch_size, self.shape[0]))
            self.used_gpu_memory += self.dy.shape[0] * self.dy.shape[1] * 4
        Edge.to_gpu(self, batch_size)

    def from_gpu(self):
        if self.on_gpu:
            self.o.free_device_memory()
            self.used_gpu_memory -= self.o.shape[0] * self.o.shape[1] * 4
            self.dy.free_device_memory()
            self.used_gpu_memory -= self.dy.shape[0] * self.dy.shape[1] * 4
            del self.o
        Edge.from_gpu(self)

    def init_training(self, batch_size):
        self.to_gpu(batch_size)
        self.W.assign(1.)
        self.scale = 1.

    def finish_training(self):
        self.from_gpu()

    def up(self, nodes):
        (node1, node2) = (nodes[self.conn[0]], nodes[self.conn[1]])
        cm_conv.MaxPool(node1.y, self.o, self.n_channel, self.ratio, 0,
                        self.stride, node2.shape[0])
        node2.y.assign(self.o)
        node2.y.mult(self.scale)

    def down(self, nodes):
        (node1, node2) = (nodes[self.conn[0]], nodes[self.conn[1]])
        cm_conv.MaxPoolUndo(
            node1.y, self.dy, node2.dy, self.o, self.ratio, 0, self.stride,
            node2.shape[0])
        node1.dy.add_mult(self.dy, self.scale)

    def gradient(self, nodes, gW):
        gW.assign(0)
        self.scale = float(self.W.asarray()[0])

    def to_dict(self):
        """Convert self to a dict."""

        result = Edge.to_dict(self)
        result["ratio"] = self.ratio
        result["stride"] = self.stride
        result["scale"] = self.scale
        return result



import os
import unittest

import requests
import numpy as np

from mang.feedforwardnetwork import FeedForwardNetwork as FF
from mang import preprocessing as prep
from mang import measure


class TestFeedForwardNetwork(unittest.TestCase):
    """Test cases for FeedForwardNetwork class."""

    def setUp(self):
        """Prepare MNIST dataset for training."""

        # download mnist test dataset from remote server
        script_dir = os.path.dirname(__file__)
        mnist_path = os.path.join(script_dir, 'mnist.npz')
        if not os.path.exists(mnist_path):
            print "downloading mnist test dataset..."
            mnist_url = 'http://db.tt/D5E3GTvR'
            r = requests.get(mnist_url)
            fp = open(mnist_path, 'w')
            fp.write(r.content)
            fp.close()
            print "done!"

        data = np.load(mnist_path)
        self.data = {
            'input': prep.zca_whiten(data['X'])[0],
            'output': np.array(data['label']),
            }

        self.nodes = {
            'input': {
                'type': 'affine', 'shape': (28, 28, 1), 'use_bias': False, },
            'conv1': {
                'type': 'relu', 'shape': (24, 24, 16), 'shared': True, },
            'pool1': {
                'type': 'rnorm', 'shape': (12, 12, 16), 'norm_size': 2,
                'add_scale': 1., 'pow_scale': .75, },
            'conv2': {'type': 'relu', 'shape': (8, 8, 64), 'shared': True, },
            'pool2': {
                'type': 'rnorm', 'shape': (4, 4, 64), 'norm_size': 2,
                'add_scale': 1., 'pow_scale': .75, },
            'hidden': {'type': 'relu', 'shape': (500, )},
            'output': {'type': 'softmax', 'shape': (10, )},
            }

        self.edges = {
            ('input', 'conv1'): {
                'type': 'conv', 'filter_size': 5, 'padding': 0, 'stride': 1,
                },
            ('conv1', 'pool1'): {'type': 'max_pool', 'ratio': 2, },
            ('pool1', 'conv2'): {
                'type': 'conv', 'filter_size': 5, 'padding': 0, 'stride': 1,
                },
            ('conv2', 'pool2'): {'type': 'max_pool', 'ratio': 2, },
            ('pool2', 'hidden'): {'type': 'full'},
            ('hidden', 'output'): {'type': 'full'},
            }

    def test_convnet(self):
        """Test if network is trained properly."""

        net = FF(self.nodes, self.edges)

        node_param = {
            'conv1': {'init_b': 0., 'eps': 1e-1, },
            'conv2': {'init_b': 0., 'eps': 1e-1, },
            'hidden': {'init_b': 0., 'eps': 1e-1, },
            'output': {'cost': 'squared_error', },
            }
        edge_param = {
            ('input', 'conv1'): {'eps': 1e-1, },
            ('pool1', 'conv2'): {'eps': 1e-1, },
            ('pool2', 'hidden'): {'eps': 1e-1, },
            ('hidden', 'output'): {'eps': 1e-1, },
            }
        net.fit(self.data, n_epoch=10, batch_size=128,
                node_param=node_param, edge_param=edge_param)
        result = net.evaluate(self.data, {'output': 'accuracy', })
        self.assertTrue(result['output'] > 0.9)

    def test_save_load(self):
        """Test save/load functionality."""

        net = FF(self.nodes, self.edges)
        net.fit(self.data, n_epoch=2)
        output = net.feed_forward(self.data)['output']

        net.save('tmp.msgpack')
        net_recovered = FF.load('tmp.msgpack')
        os.system('rm tmp.msgpack')
        output_recovered = net_recovered.feed_forward(self.data)['output']

        conn_list = [
            ('input', 'conv1'),
            ('pool2', 'hidden'),
            ('hidden', 'output'),
            ]
        for conn in conn_list:
            w_original = net.edges[conn].W
            w_recovered = net_recovered.edges[conn].W
            self.assertTrue(np.allclose(w_original, w_recovered))

        for name in ['conv1', 'conv2', 'hidden', 'output']:
            b_original = net.nodes[name].b
            b_recovered = net_recovered.nodes[name].b
            self.assertTrue(np.allclose(w_original, w_recovered))
            self.assertTrue(np.allclose(b_original, b_recovered))

        print abs(output - output_recovered).mean()
        self.assertTrue(np.allclose(output, output_recovered))

    def test_shared_edges(self):
        """Test shared edges."""

        nodes = {
            'input': {'type': 'affine', 'shape': (28, 28), },
            'hidden': {'type': 'relu', 'shape': (500, ), },
            'output': {'type': 'affine', 'shape': (28, 28), },
            }
        edges = {
            ('input', 'hidden'): {'type': 'full', },
            ('hidden', 'output'): {
                'type': 'ref',
                'original': ('input', 'hidden'),
                'transpose': True,
                },
            }
        node_param = {
            'input': {'noise': 'pepper', 'noise_param': .2, }
            }

        net = FF(nodes, edges)

        data = {'input': self.data['input'], 'output': self.data['input'], }
        net.fit(data, n_epoch=2, node_param=node_param)
        net.feed_forward(data)

        net.save('tmp.msgpack')
        net_recovered = FF.load('tmp.msgpack')
        os.system('rm tmp.msgpack')
        net_recovered.feed_forward(data)


if __name__ == '__main__':
    unittest.main()



import pdb
import numpy as np
import nose
import cudamat as cm

def setup():
    cm.cublas_init()

def teardown():
    cm.cublas_shutdown()

def test_reshape():
    m = 256
    n = 1
    cm1 = np.array(np.random.rand(n, m)*10, dtype=np.float32, order='F')
    cm2 = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')

    gm1 = cm.CUDAMatrix(cm1)
    gm2 = cm.CUDAMatrix(cm2)

    gm1.reshape((m, n))
    gm2.assign(gm1)
    gm1.reshape((n, m))

    gm1.copy_to_host()
    gm2.copy_to_host()

    assert np.max(np.abs(gm1.numpy_array - gm2.numpy_array.T)) < 10**-2, "Error in CUDAMatrix.reshape exceeded threshold"

def test_T_field():
    m = 256
    n = 128
    cm1 = np.array(np.random.rand(n, m)*10, dtype=np.float32, order='F')
    cm2 = np.array(np.random.rand(m, 1)*10, dtype=np.float32, order='F')
    gm1 = cm.CUDAMatrix(cm1)
    gm2 = cm.CUDAMatrix(cm2)

    # test dot
    gm = cm.dot(gm2.T, gm1.T)
    c = np.dot(cm2.T, cm1.T)
    gm.copy_to_host()

    assert np.max(np.abs(gm.numpy_array - c)) < 10**-2, "Error in CUDAMatrix.dot with TransposedCUDAMatrix exceeded threshold"

    # test add_dot
    cm3 = np.array(np.random.rand(1, n)*10, dtype=np.float32, order='F')
    gm3 = cm.CUDAMatrix(cm3)
    gm3.add_dot(gm2.T, gm1.T)
    c = cm3 + np.dot(cm2.T, cm1.T)
    gm3.copy_to_host()

    assert np.max(np.abs(gm3.numpy_array - c)) < 10**-2, "Error in CUDAMatrix.add_dot TransposedCUDAMatrix exceeded threshold"

    # test add_sums
    gm2.add_sums(gm1.T, axis = 1)
    c = cm2 + np.atleast_2d(cm1.sum(0)).T
    gm2.copy_to_host()

    assert np.max(np.abs(gm2.numpy_array - c)) < 10**-2, "Error in CUDAMatrix.add_sums TransposedCUDAMatrix exceeded threshold"

def test_assign():
    m = 256
    n = 128
    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    
    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)

    m1.assign(m2)
    m1.copy_to_host()

    assert np.max(np.abs(m1.numpy_array - m2.numpy_array)) < 10**-4, "Error in CUDAMatrix.assign exceeded threshold"

def test_assign_scalar():
    m = 256
    n = 128
    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    
    m1 = cm.CUDAMatrix(a)

    m1.assign(np.pi)
    m1.copy_to_host()

    assert np.max(np.abs(m1.numpy_array - np.pi)) < 10**-4, "Error in CUDAMatrix.assign_scalar exceeded threshold"

def test_get_row_slice():
    m = 256
    n = 128
    start = 11
    end = 54

    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.rand(end-start, n)*10, dtype=np.float32, order='F')
    
    c = np.array(a[start:end,:], order='F')
    
    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m1.get_row_slice(start, end, target = m2)
    m3 = m1.get_row_slice(start, end)
    m2.copy_to_host()
    m3.copy_to_host()

    #pdb.set_trace()
    assert np.max(np.abs(c - m2.numpy_array)) < 10**-4, "Error in CUDAMatrix.get_row_slice exceeded threshold"
    assert np.max(np.abs(c - m3.numpy_array)) < 10**-4, "Error in CUDAMatrix.get_row_slice exceeded threshold"

def test_set_row_slice():
    m = 256
    n = 128
    start = 11
    end = 54

    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.rand(end-start, n)*10, dtype=np.float32, order='F')
    
    c = a.copy()
    c[start:end,:] = b
    
    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m1.set_row_slice(start, end, m2)
    m1.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.set_row_slice exceeded threshold"

def test_transpose():
    m = 6
    n = 128

    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.rand(n, m), dtype=np.float32, order='F')
    
    c = a.copy().T
    
    m = cm.CUDAMatrix(a)
    mt1 = cm.CUDAMatrix(b)
    m.transpose(target = mt1)
    mt2 = m.transpose()

    mt1.copy_to_host()
    mt2.copy_to_host()

    assert np.max(np.abs(c - mt1.numpy_array)) < 10**-4, "Error in CUDAMatrix.transpose exceeded threshold"
    assert np.max(np.abs(c - mt2.numpy_array)) < 10**-4, "Error in CUDAMatrix.transpose exceeded threshold"

def test_slice():
    m = 256
    n = 128
    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    
    c = np.array(a[:,32:64], order='F')
    
    m1 = cm.CUDAMatrix(a)
    m2 = m1.slice(32, 64)
    m2.copy_to_host()

    assert np.max(np.abs(c - m2.numpy_array)) < 10**-4, "Error in CUDAMatrix.slice exceeded threshold"


def test_add_col_vec():
    m = 250
    n = 120
    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.rand(m, 1)*10, dtype=np.float32, order='F')
    t = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    
    c = a + b
    
    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m3 = cm.CUDAMatrix(t)

    m1.add_col_vec(m2, target = m3)
    m1.add_col_vec(m2)
    m1.copy_to_host()
    m3.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.add_col_vec exceeded threshold"
    assert np.max(np.abs(c - m3.numpy_array)) < 10**-4, "Error in CUDAMatrix.add_col_vec exceeded threshold"

def test_add_col_mult():
    m = 256
    n = 128
    mult = np.pi
    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.rand(m, 1)*10, dtype=np.float32, order='F')
    t = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    
    c = a + mult * b
    
    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m3 = cm.CUDAMatrix(t)

    m1.add_col_mult(m2, mult, target = m3)
    m1.add_col_mult(m2, mult)
    m1.copy_to_host()
    m3.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.add_col_mult exceeded threshold"
    assert np.max(np.abs(c - m3.numpy_array)) < 10**-4, "Error in CUDAMatrix.add_col_mult exceeded threshold"

def test_add_row_vec():
    m = 256
    n = 128
    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.rand(1, n)*10, dtype=np.float32, order='F')
    t = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    
    c = a + b
    
    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m3 = cm.CUDAMatrix(t)

    m1.add_row_vec(m2, target = m3)
    m1.add_row_vec(m2)
    m1.copy_to_host()
    m3.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.add_row_vec exceeded threshold"
    assert np.max(np.abs(c - m3.numpy_array)) < 10**-4, "Error in CUDAMatrix.add_row_vec exceeded threshold"

def test_mult_by_col():
    m = 256
    n = 128
    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.rand(m, 1)*10, dtype=np.float32, order='F')
    t = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    
    c = a * b
    
    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m3 = cm.CUDAMatrix(t)

    m1.mult_by_col(m2, target = m3)
    m1.mult_by_col(m2)
    m1.copy_to_host()
    m3.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.mult_by_col exceeded threshold"
    assert np.max(np.abs(c - m3.numpy_array)) < 10**-4, "Error in CUDAMatrix.mult_by_col exceeded threshold"

def test_mult_by_row():
    m = 256
    n = 128
    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.rand(1, n)*10, dtype=np.float32, order='F')
    t = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    
    c = a * b
    
    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m3 = cm.CUDAMatrix(t)

    m1.mult_by_row(m2, target = m3)
    m1.mult_by_row(m2)
    m1.copy_to_host()
    m3.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.mult_by_row exceeded threshold"
    assert np.max(np.abs(c - m3.numpy_array)) < 10**-4, "Error in CUDAMatrix.mult_by_row exceeded threshold"

def test_sum():
    m = 256
    n = 128
    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    t1 = np.array(np.random.rand(1, n)*10, dtype=np.float32, order='F')
    t2 = np.array(np.random.rand(m, 1)*10, dtype=np.float32, order='F')
    
    c1 = np.atleast_2d(a.sum(0))
    c2 = np.atleast_2d(a.sum(1)).T
    
    m = cm.CUDAMatrix(a)
    mt1 = cm.CUDAMatrix(t1)
    mt2 = cm.CUDAMatrix(t2)

    m.sum(axis = 0, target = mt1)
    mt1r = m.sum(axis = 0)

    m.sum(axis = 1, target = mt2)
    mt2r = m.sum(axis = 1)

    mt1.copy_to_host()
    mt1r.copy_to_host()
    mt2.copy_to_host()
    mt2r.copy_to_host()

    assert np.max(np.abs(c1 - mt1.numpy_array)) < 10**-3, "Error in CUDAMatrix.sum exceeded threshold"
    assert np.max(np.abs(c1 - mt1r.numpy_array)) < 10**-3, "Error in CUDAMatrix.sum exceeded threshold"
    assert np.max(np.abs(c2 - mt2.numpy_array)) < 10**-3, "Error in CUDAMatrix.sum exceeded threshold"
    assert np.max(np.abs(c2 - mt2r.numpy_array)) < 10**-3, "Error in CUDAMatrix.sum exceeded threshold"

def test_sum_trans():
    m = 256
    n = 128
    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    t1 = np.array(np.random.rand(1, m)*10, dtype=np.float32, order='F')
    t2 = np.array(np.random.rand(n, 1)*10, dtype=np.float32, order='F')
    
    c1 = np.atleast_2d(a.T.sum(0))
    c2 = np.atleast_2d(a.T.sum(1)).T
    
    m = cm.CUDAMatrix(a)
    m.set_trans(True)
    mt1 = cm.CUDAMatrix(t1)
    mt2 = cm.CUDAMatrix(t2)

    m.sum(axis = 0, target = mt1)
    mt1r = m.sum(axis = 0)

    m.sum(axis = 1, target = mt2)
    mt2r = m.sum(axis = 1)

    mt1.copy_to_host()
    mt1r.copy_to_host()
    mt2.copy_to_host()
    mt2r.copy_to_host()

    assert np.max(np.abs(c1 - mt1.numpy_array)) < 10**-3, "Error in CUDAMatrix.sum exceeded threshold"
    assert np.max(np.abs(c1 - mt1r.numpy_array)) < 10**-3, "Error in CUDAMatrix.sum exceeded threshold"
    assert np.max(np.abs(c2 - mt2.numpy_array)) < 10**-3, "Error in CUDAMatrix.sum exceeded threshold"
    assert np.max(np.abs(c2 - mt2r.numpy_array)) < 10**-3, "Error in CUDAMatrix.sum exceeded threshold"

def test_add_sums():
    m = 256
    n = 128

    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    t1 = np.array(np.random.rand(m, 1)*10, dtype=np.float32, order='F')
    t2 = np.array(np.random.rand(1, n)*10, dtype=np.float32, order='F')

    mult = np.pi
    
    c1 = t1 + mult * np.atleast_2d(a.sum(1)).T
    c2 = t2 + np.atleast_2d(a.sum(0))
    
    m = cm.CUDAMatrix(a)
    mt1 = cm.CUDAMatrix(t1)
    mt2 = cm.CUDAMatrix(t2)

    mt1.add_sums(m, axis = 1, mult = np.pi)
    mt2.add_sums(m, axis = 0)

    mt1.copy_to_host()
    mt2.copy_to_host()

    assert np.max(np.abs(c1 - mt1.numpy_array)) < 10**-3, "Error in CUDAMatrix.add_sums exceeded threshold"
    assert np.max(np.abs(c2 - mt2.numpy_array)) < 10**-3, "Error in CUDAMatrix.add_sums exceeded threshold"


def test_less_than():
    m = 256
    n = 128
    a = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    t1 = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    t2 = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    v = 0.1

    r1 = 1 * (a < b)
    r2 = 1 * (a < v)

    da = cm.CUDAMatrix(a)
    db = cm.CUDAMatrix(b)
    dt1 = cm.CUDAMatrix(t1)
    dt2 = cm.CUDAMatrix(t2)

    da.less_than(db, target = dt1)
    da.less_than(v, target = dt2)
    da.less_than(db)

    da.copy_to_host()
    dt1.copy_to_host()
    dt2.copy_to_host()

    assert np.max(np.abs(r1 - da.numpy_array)) < 10**-4, "Error in CUDAMatrix.less_than exceeded threshold"
    assert np.max(np.abs(r1 - dt1.numpy_array)) < 10**-4, "Error in CUDAMatrix.less_than exceeded threshold"
    assert np.max(np.abs(r2 - dt2.numpy_array)) < 10**-4, "Error in CUDAMatrix.less_than exceeded threshold"

def test_greater_than():
    m = 256
    n = 128
    a = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    t1 = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    t2 = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    v = 0.1

    r1 = 1 * (a > b)
    r2 = 1 * (a > v)

    da = cm.CUDAMatrix(a)
    db = cm.CUDAMatrix(b)
    dt1 = cm.CUDAMatrix(t1)
    dt2 = cm.CUDAMatrix(t2)

    da.greater_than(db, target = dt1)
    da.greater_than(v, target = dt2)
    da.greater_than(db)

    da.copy_to_host()
    dt1.copy_to_host()
    dt2.copy_to_host()

    assert np.max(np.abs(r1 - da.numpy_array)) < 10**-4, "Error in CUDAMatrix.greater_than exceeded threshold"
    assert np.max(np.abs(r1 - dt1.numpy_array)) < 10**-4, "Error in CUDAMatrix.greater_than exceeded threshold"
    assert np.max(np.abs(r2 - dt2.numpy_array)) < 10**-4, "Error in CUDAMatrix.greater_than exceeded threshold"

def test_max():
    m = 256
    n = 128
    a = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    t = np.array(np.random.rand(1, n)*10, dtype=np.float32, order='F')
   
    r = np.atleast_2d(a.max(0)) 
    
    da = cm.CUDAMatrix(a)
    dr1 = cm.CUDAMatrix(t)

    da.max(axis = 0, target = dr1)
    dr2 = da.max(axis = 0)

    dr1.copy_to_host()
    dr2.copy_to_host()

    assert np.max(np.abs(r - dr1.numpy_array)) < 10**-4, "Error in CUDAMatrix.max exceeded threshold"
    assert np.max(np.abs(r - dr2.numpy_array)) < 10**-4, "Error in CUDAMatrix.max exceeded threshold"

def test_max2():
    m = 256
    n = 128
    a = np.array(-np.random.rand(m, n)*10, dtype=np.float32, order='F')
    t = np.array(np.random.rand(1, n)*10, dtype=np.float32, order='F')
   
    r = np.atleast_2d(a.max(0)) 
    
    da = cm.CUDAMatrix(a)
    dr1 = cm.CUDAMatrix(t)

    da.max(axis = 0, target = dr1)
    dr2 = da.max(axis = 0)

    dr1.copy_to_host()
    dr2.copy_to_host()

    assert np.max(np.abs(r - dr1.numpy_array)) < 10**-4, "Error in CUDAMatrix.max exceeded threshold"
    assert np.max(np.abs(r - dr2.numpy_array)) < 10**-4, "Error in CUDAMatrix.max exceeded threshold"

def test_sign():
    m = 256
    n = 128
    a = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    a[0,0] = 0.
    a[0,1] = -0.
    t = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')

    c = np.sign(a)

    m1 = cm.CUDAMatrix(a)
    m3 = cm.CUDAMatrix(t)

    m2 = m1.sign()
    m1.sign(target = m3)

    m2.copy_to_host()
    m3.copy_to_host()

    assert np.max(np.abs(c - m2.numpy_array)) < 10**-4, "Error in CUDAMatrix.sign exceeded threshold"
    assert np.max(np.abs(c - m3.numpy_array)) < 10**-4, "Error in CUDAMatrix.sign exceeded threshold"

def test_sigmoid():
    m = 256
    n = 128
    a = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')

    c = 1. / (1. + np.exp(-a))

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m1.apply_sigmoid(target = m2)
    m1.apply_sigmoid()

    m1.copy_to_host()
    m2.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.apply_sigmoid exceeded threshold"
    assert np.max(np.abs(c - m2.numpy_array)) < 10**-4, "Error in CUDAMatrix.apply_sigmoid exceeded threshold"

def test_log():
    m = 256
    n = 128
    a = np.array(np.random.rand(m, n)*10+0.1, dtype=np.float32, order='F')
    b = np.array(np.random.rand(m, n)*10+0.1, dtype=np.float32, order='F')

    c = np.log(a)

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    cm.log(m1, target = m2)
    cm.log(m1)

    m1.copy_to_host()
    m2.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in cudamat.log exceeded threshold"
    assert np.max(np.abs(c - m2.numpy_array)) < 10**-4, "Error in cudamat.log exceeded threshold"

def test_exp():
    m = 256
    n = 128
    a = np.array(np.random.randn(m, n), dtype=np.float32, order='F')
    b = np.array(np.random.randn(m, n), dtype=np.float32, order='F')

    c = np.exp(a)

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    cm.exp(m1, target = m2)
    cm.exp(m1)

    m1.copy_to_host()
    m2.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in cudamat.exp exceeded threshold"
    assert np.max(np.abs(c - m2.numpy_array)) < 10**-4, "Error in cudamat.exp exceeded threshold"

def test_sqrt():
    m = 256
    n = 128
    a = np.array(np.random.rand(m, n)*20, dtype=np.float32, order='F')
    b = np.array(np.random.rand(m, n), dtype=np.float32, order='F')

    c = np.sqrt(a)

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    cm.sqrt(m1, target = m2)
    cm.sqrt(m1)

    m1.copy_to_host()
    m2.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in cudamat.sqrt exceeded threshold"
    assert np.max(np.abs(c - m2.numpy_array)) < 10**-4, "Error in cudamat.sqrt exceeded threshold"

def test_pow():
    m = 256
    n = 128
    a = np.array(np.random.randn(m, n)*20, dtype=np.float32, order='F')
    b = np.array(np.random.rand(m, n), dtype=np.float32, order='F')
    p = 2

    c = a**p

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    cm.pow(m1, p, target = m2)
    cm.pow(m1, p)

    m1.copy_to_host()
    m2.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-3, "Error in cudamat.pow exceeded threshold"
    assert np.max(np.abs(c - m2.numpy_array)) < 10**-3, "Error in cudamat.pow exceeded threshold"

def test_pow_matrix():
    m = 256
    n = 128
    a = np.array(np.random.rand(m, n)*20, dtype=np.float32, order='F')
    b = np.array(np.random.rand(m, n), dtype=np.float32, order='F')
    p = np.array(np.random.randn(m, n), dtype=np.float32, order='F')


    c = a**p

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    mp = cm.CUDAMatrix(p)
    cm.pow(m1, mp, target = m2)
    cm.pow(m1, mp)

    m1.copy_to_host()
    m2.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-2, "Error in cudamat.pow exceeded threshold"
    assert np.max(np.abs(c - m2.numpy_array)) < 10**-2, "Error in cudamat.pow exceeded threshold"

def test_reciprocal():
    m = 256
    n = 128
    a = np.array(np.random.rand(m, n)*10+10**-3, dtype=np.float32, order='F')
    b = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')

    c = 1. / a

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m1.reciprocal(target = m2)
    m1.reciprocal()

    m1.copy_to_host()
    m2.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.reciprocal exceeded threshold"
    assert np.max(np.abs(c - m2.numpy_array)) < 10**-4, "Error in CUDAMatrix.reciprocal exceeded threshold"

def test_add_mult():
    m = 256
    n = 128
    alpha = np.pi
    a = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')

    c = a + np.pi * b

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m1.add_mult(m2, np.pi)
    m1.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.add_mult exceeded threshold"

def test_subtract_mult():
    m = 256
    n = 128
    alpha = np.pi
    a = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')

    c = a - np.pi * b

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m1.subtract_mult(m2, np.pi)
    m1.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.subtract_mult exceeded threshold"

def test_add():
    m = 256
    n = 128
    a = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    b = np.array(1.+np.random.rand(m, n)*10, dtype=np.float32, order='F')
    t = np.array(np.empty((m, n)), dtype=np.float32, order='F')

    c = a + b

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m3 = cm.CUDAMatrix(t)

    m1.add(m2, target = m3)
    m1.add(m2)

    m3.copy_to_host()
    m1.copy_to_host()

    assert np.max(np.abs(c - m3.numpy_array)) < 10**-4, "Error in CUDAMatrix.add exceeded threshold"
    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.add exceeded threshold"

def test_subtract():
    m = 256
    n = 128
    a = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    b = np.array(1.+np.random.rand(m, n)*10, dtype=np.float32, order='F')
    t = np.array(np.empty((m, n)), dtype=np.float32, order='F')

    c = a - b

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m3 = cm.CUDAMatrix(t)

    m1.subtract(m2, target = m3)
    m1.subtract(m2)

    m3.copy_to_host()
    m1.copy_to_host()

    assert np.max(np.abs(c - m3.numpy_array)) < 10**-4, "Error in CUDAMatrix.subtract exceeded threshold"
    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.subtract exceeded threshold"

def test_divide():
    m = 256
    n = 128
    a = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    b = np.array(1.+np.random.rand(m, n)*10, dtype=np.float32, order='F')
    t = np.array(np.empty((m, n)), dtype=np.float32, order='F')

    c = a / b

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m3 = cm.CUDAMatrix(t)

    m1.divide(m2, target = m3)
    m1.divide(m2)

    m3.copy_to_host()
    m1.copy_to_host()

    assert np.max(np.abs(c - m3.numpy_array)) < 10**-4, "Error in CUDAMatrix.div exceeded threshold"
    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.div exceeded threshold"

def test_mult():
    m = 256
    n = 128
    a = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    b = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    t = np.array(np.empty((m, n)), dtype=np.float32, order='F')

    c = a * b

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m3 = cm.CUDAMatrix(t)

    m1.mult(m2, target = m3)
    m1.mult(m2)

    m3.copy_to_host()
    m1.copy_to_host()

    assert np.max(np.abs(c - m3.numpy_array)) < 10**-4, "Error in CUDAMatrix.multiply exceeded threshold"
    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.multiply exceeded threshold"

def test_scalar_mult():
    m = 256
    n = 128
    alpha = np.pi
    a = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    t = np.array(np.empty((m, n)), dtype=np.float32, order='F')

    c = a * alpha

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(t)

    m1.mult(alpha, target = m2)
    m1.mult(alpha)

    m1.copy_to_host()
    m2.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.mult exceeded threshold"
    assert np.max(np.abs(c - m2.numpy_array)) < 10**-4, "Error in CUDAMatrix.mult exceeded threshold"

def test_scalar_div():
    m = 256
    n = 128
    alpha = np.pi
    a = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    t = np.array(np.empty((m, n)), dtype=np.float32, order='F')

    c = a / alpha

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(t)

    m1.divide(alpha, target = m2)
    m1.divide(alpha)

    m1.copy_to_host()
    m2.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.divide exceeded threshold"
    assert np.max(np.abs(c - m2.numpy_array)) < 10**-4, "Error in CUDAMatrix.divide exceeded threshold"

def test_add_scalar():
    m = 256
    n = 128
    alpha = np.pi
    a = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    t = np.array(np.empty((m, n)), dtype=np.float32, order='F')

    c = a + alpha

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(t)

    m1.add(alpha, target = m2)
    m1.add(alpha)

    m1.copy_to_host()
    m2.copy_to_host()

    assert np.max(np.abs(c - m1.numpy_array)) < 10**-4, "Error in CUDAMatrix.add_scalar exceeded threshold"
    assert np.max(np.abs(c - m2.numpy_array)) < 10**-4, "Error in CUDAMatrix.add_scalar exceeded threshold"

def test_dot():
    m = 128
    k = 256
    n = 64
    a = np.array(np.random.randn(m, k)*10, dtype=np.float32, order='F')
    b = np.array(np.random.randn(k, n)*10, dtype=np.float32, order='F')

    c = np.dot(a, b)

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m3 = cm.dot(m1, m2)
    m3.copy_to_host()

    assert np.max(np.abs(c - m3.numpy_array)) < 10**-2, "Error in CUDAMatrix.dot exceeded threshold"

def test_dot_trans():
    m = 128
    k = 256
    n = 64
    a = np.array(np.random.randn(k, m)*10, dtype=np.float32, order='F')
    b = np.array(np.random.randn(k, n)*10, dtype=np.float32, order='F')

    c = np.dot(a.T, b)

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m1.set_trans(True);
    m3 = cm.dot(m1, m2)
    m3.copy_to_host()

    assert np.max(np.abs(c - m3.numpy_array)) < 10**-2, "Error in CUDAMatrix.dot exceeded threshold"

def test_add_dot():
    m = 128
    k = 256
    n = 64
    a = np.array(np.random.randn(m, k)*10, dtype=np.float32, order='F')
    b = np.array(np.random.randn(k, n)*10, dtype=np.float32, order='F')
    c = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')

    res = c + np.dot(a, b)

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m3 = cm.CUDAMatrix(c)
    m3.add_dot(m1, m2)

    m3.copy_to_host()

    assert np.max(np.abs(res - m3.numpy_array)) < 10**-2, "Error in CUDAMatrix.add_dot exceeded threshold"

def test_vdot():
    m = 64
    n = 64
    a = np.array(np.random.randn(m, n), dtype=np.float32, order='F')
    b = np.array(np.random.randn(m, n), dtype=np.float32, order='F')

    true_res = np.vdot(a, b)

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)

    res = cm.vdot(m1, m2)

    assert np.abs(res - true_res) < 10**-2, "Error in CUDAMatrix.vdot exceeded threshold"

def test_subtract_dot():
    m = 128
    k = 256
    n = 64
    a = np.array(np.random.randn(m, k)*10, dtype=np.float32, order='F')
    b = np.array(np.random.randn(k, n)*10, dtype=np.float32, order='F')
    c = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')

    res = c - np.dot(a, b)

    m1 = cm.CUDAMatrix(a)
    m2 = cm.CUDAMatrix(b)
    m3 = cm.CUDAMatrix(c)
    m3.subtract_dot(m1, m2)

    m3.copy_to_host()

    assert np.max(np.abs(res - m3.numpy_array)) < 10**-2, "Error in CUDAMatrix.subtract_dot exceeded threshold"

def test_random():
    cm.CUDAMatrix.init_random(1)
    m1 = cm.CUDAMatrix(np.array(np.empty((128,256)), dtype=np.float32, order='F'))
    m2 = cm.CUDAMatrix(np.array(np.empty((128,256)), dtype=np.float32, order='F'))

    m1.fill_with_rand()
    m1.copy_to_host()
    m2.fill_with_randn()
    m2.copy_to_host()

    assert np.abs(np.mean(m1.numpy_array) - 0.5) < 10**-2, "Error in CUDAMatrix.fill_with_rand threshold"
    assert np.abs(np.mean(m2.numpy_array)) < 10**-2, "Error in CUDAMatrix.fill_with_randn threshold"

def test_euclid_norm():
    m = 256
    n = 128
    a = np.array(np.random.rand(m, n)*10, dtype=np.float32, order='F')
    
    m = cm.CUDAMatrix(a)

    n1 = np.sqrt(np.sum(a**2))
    n2 = m.euclid_norm()

    assert np.abs(n1-n2) < 10**-2, "Error in CUDAMatrix.euclid_norm exceeded threshold"

def test_select_columns():
    m = 256
    n = 128
    k = 8

    s = np.array(np.random.randn(m, n), dtype=np.float32, order='F')
    i_l = [0, 1, 2, 3, 5, 10, 12, 20]
    i = np.array(i_l).T[np.newaxis, :]
    t = np.empty((m, k))
    
    s_d = cm.CUDAMatrix(s)
    i_d = cm.CUDAMatrix(i)
    t_d = cm.CUDAMatrix(t)

    s_d.select_columns(i_d, t_d)
    res = s[:,i_l]

    assert np.max(np.abs(res - t_d.asarray())) < 10**-4, "Error in CUDAMatrix.select_columns exceeded threshold"

if __name__ == '__main__':
    nose.run()



from operator import mul

import numpy as np

import mang.cudamat as cm


class Edge(object):

    _name = "full"

    def __init__(self, nodes, conn, option={}):
        self.conn = conn
        size_in = reduce(mul, nodes[conn[0]].shape)
        size_out = reduce(mul, nodes[conn[1]].shape)
        self.shape = (size_in, size_out)
        self.W = option["W"] if "W" in option else np.zeros(self.shape)
        self.on_gpu = False
        self.used_gpu_memory = 0

    def to_gpu(self, batch_size):
        if not self.on_gpu:
            self.W = cm.CUDAMatrix(self.W)
            self.used_gpu_memory += self.W.shape[0] * self.W.shape[1] * 4
            self.on_gpu = True

    def from_gpu(self):
        if self.on_gpu:
            W = self.W.asarray()
            self.W.free_device_memory()
            self.used_gpu_memory -= self.W.shape[0] * self.W.shape[1] * 4
            self.W = W
            self.on_gpu = False

    def init_training(self, option):
        self.to_gpu(option)

    def finish_training(self):
        self.from_gpu()

    def up(self, nodes):
        (node1, node2) = (nodes[self.conn[0]], nodes[self.conn[1]])
        node2.y.add_dot(node1.y, self.W)

    def down(self, nodes):
        (node1, node2) = (nodes[self.conn[0]], nodes[self.conn[1]])
        node1.dy.add_dot(node2.dy, self.W.T)

    def gradient(self, nodes, gW):
        (node1, node2) = (nodes[self.conn[0]], nodes[self.conn[1]])
        gW.add_dot(node1.y.T, node2.dy, 1. / node1.y.shape[0])

    def to_dict(self):
        """Convert self into a dict."""

        W = self.W.asarray() if self.on_gpu else self.W
        result = {
            "type": self._name,
            "conn": self.conn,
            "W": W,
            }
        return result

    @classmethod
    def from_dict(cls, nodes, data):
        """Create an edge object from a dict."""

        return cls(nodes, data["conn"], data)



from operator import mul

import numpy as np

import mang.cudamat as cm


class Node(object):
    """Node with linear(or identity) activation function."""
    _name = "affine"

    def __init__(self, shape, option):
        self.shape = shape
        self.size = reduce(mul, self.shape)

        if "use_bias" in option:
            self.use_bias = option["use_bias"]
        else:
            self.use_bias = True

        if self.use_bias:
            if "shared" in option and option["shared"]:
                self.b = np.zeros((self.shape[-1], ))
                self.shared = True
            else:
                self.b = np.zeros((self.size, ))
                self.shared = False
        else:
            self.b = None

        if "b" in option:
            self.b = np.array(option["b"], dtype=np.float32, order="F")
            self.use_bias = True

        self.on_gpu = False
        self.used_gpu_memory = 0

    def to_gpu(self, batch_size):
        """Initialize GPU variables."""

        if not self.on_gpu:
            if self.use_bias:
                self.b = cm.CUDAMatrix(self.b.reshape(1, self.b.size))
                self.used_gpu_memory += 4 * self.b.shape[0] * self.b.shape[1]
            self.y = cm.empty((batch_size, self.size))
            self.used_gpu_memory += 4 * self.y.shape[0] * self.y.shape[1]

            self.batch_size = batch_size
            self.on_gpu = True

    def from_gpu(self):
        """Copy GPU variables to CPU and free allocated GPU memory."""

        if self.on_gpu:
            if self.use_bias:
                b = self.b.asarray()
                self.b.free_device_memory()
                self.b = b.reshape((b.size, ))
                self.used_gpu_memory -= 4 * self.b.size
            self.y.free_device_memory()
            self.used_gpu_memory -= self.y.shape[0] * self.y.shape[1] * 4

            del self.batch_size
            self.on_gpu = False

    def init_training(self, option):
        """Create temporary GPU variables."""

        self.to_gpu(option["batch_size"])
        self.dy = cm.empty((self.batch_size, self.size))
        self.used_gpu_memory += self.dy.shape[0] * self.dy.shape[1] * 4

    def finish_training(self):
        """Copy GPU variables to host, and free allocated GPU memory."""

        self.from_gpu()
        self.dy.free_device_memory()
        self.used_gpu_memory -= self.dy.shape[0] * self.dy.shape[1] * 4
        del self.dy

    def _make_shape(self):
        """Reshape matrix variables to add shared bias without problem."""

        if self.shared:
            n_elem = self.batch_size * self.size
            shape_new = (n_elem / self.shape[-1], self.shape[-1])
            if self.on_gpu:
                self.y = self.y.reshape(shape_new)
            else:
                self.y.reshape(shape_new)

    def _recover_shape(self):
        """Recover the shape of matrix variables reshaped by _make_shape."""
        if self.shared:
            shape_old = (self.batch_size, self.size)
            if self.on_gpu:
                self.y.reshape(shape_old)
            else:
                self.y = self.y.reshape(shape_old)

    def _add_b(self):
        """Add bias to input signal."""

        if self.use_bias:
            self._make_shape()
            if self.on_gpu:
                self.y.add_row_vec(self.b)
            else:
                self.y += self.b
            self._recover_shape()

    def up(self):
        """Calculate activation from input signals."""

        self._add_b()

    def down(self):
        """Calculate error to back-propagate from input error signals."""

        pass  # do nothing

    def gradient(self, gb):
        """Calculate the gradient of the bias."""

        if self.use_bias:
            if self.shared:
                shape_old = self.dy.shape
                n_elem = self.dy.shape[0] * self.dy.shape[1]
                shape_new = (n_elem / self.shape[-1], self.shape[-1])
                self.dy.reshape(shape_new)

            self.dy.sum(0, gb)  # should be changed to the sum over axis 1
            gb.divide(self.batch_size)

            if self.shared:
                self.dy.reshape(shape_old)

    def to_dict(self):
        """Convert self into a dictionary."""

        result = {
            "b": self.b,
            "shape": self.shape,
            "shared": self.shared,
            "use_bias": self.use_bias,
            "type": self._name,
            }
        return result

    @classmethod
    def from_dict(cls, data):
        """Create a node object from a dictionary."""

        return cls(data["shape"], data)



import os
import platform
import warnings
import ctypes as ct
import numpy as np

MAX_ONES = 1024*256

current_dir = os.path.dirname(os.path.realpath(__file__))

if platform.system() == 'Windows':
    dll_filename = "libcudamat.dll"
else:
    dll_filename = "libcudamat.so"
_cudamat = ct.cdll.LoadLibrary(os.path.join(current_dir, dll_filename))

_cudamat.get_last_cuda_error.restype = ct.c_char_p
_cudamat.cublas_init.restype = ct.c_int
_cudamat.cublas_shutdown.restype = ct.c_int
_cudamat.cuda_set_device.restype = ct.c_int
_cudamat.init_random.restype = ct.c_int

_cudamat.init_empty.restype = ct.c_int
_cudamat.reshape.restype = ct.c_int
_cudamat.copy_to_host.restype = ct.c_int
_cudamat.allocate_device_memory = ct.c_int
_cudamat.copy_to_device.restype = ct.c_int
_cudamat.copy_on_device.restype = ct.c_int
_cudamat.free_device_memory.restype = ct.c_int

_cudamat.get_slice.restype = ct.c_int
_cudamat.get_row_slice.restype = ct.c_int
_cudamat.set_row_slice.restype = ct.c_int
_cudamat.copy_transpose.restype = ct.c_int
_cudamat.get_vector_slice.restype = ct.c_int
_cudamat.fill_with_rand.restype = ct.c_int
_cudamat.fill_with_randn.restype = ct.c_int

_cudamat.add_col_vec.restype = ct.c_int
_cudamat.add_col_mult.restype = ct.c_int
_cudamat.add_row_mult.restype = ct.c_int
_cudamat.add_row_vec.restype = ct.c_int
_cudamat.mult_by_col_vec.restype = ct.c_int
_cudamat.mult_by_row_vec.restype = ct.c_int
_cudamat.div_by_col_vec.restype = ct.c_int
_cudamat.div_by_row_vec.restype = ct.c_int

_cudamat.less_than.restype = ct.c_int
_cudamat.less_than_scalar.restype = ct.c_int
_cudamat.greater_than.restype = ct.c_int
_cudamat.greater_than_scalar.restype = ct.c_int
_cudamat.max_by_axis.restype = ct.c_int
_cudamat.argmax_by_axis.restype = ct.c_int
_cudamat.sqsum_by_axis.restype = ct.c_int
_cudamat.normlimit_by_axis.restype = ct.c_int
_cudamat.sign.restype = ct.c_int
_cudamat.apply_sigmoid.restype = ct.c_int
_cudamat.apply_tanh.restype = ct.c_int
_cudamat.apply_abs.restype = ct.c_int
_cudamat.apply_log_1_plus_exp.restype = ct.c_int
_cudamat.apply_log.restype = ct.c_int
_cudamat.apply_floor.restype = ct.c_int
_cudamat.apply_ceil.restype = ct.c_int
_cudamat.apply_exp.restype = ct.c_int
_cudamat.apply_sqrt.restype = ct.c_int
_cudamat.apply_pow.restype = ct.c_int
_cudamat.apply_pow_matrix.restype = ct.c_int
_cudamat.reciprocal.restype = ct.c_int

_cudamat.add_elementwise.restype = ct.c_int
_cudamat.subtract_elementwise.restype = ct.c_int
_cudamat.divide_elementwise.restype = ct.c_int
_cudamat.mult_elementwise.restype = ct.c_int
_cudamat.apply_logistic_deriv.restype = ct.c_int
_cudamat.assign_scalar.restype = ct.c_int
_cudamat.mult_by_scalar.restype = ct.c_int
_cudamat.divide_by_scalar.restype = ct.c_int
_cudamat.add_scalar.restype = ct.c_int

_cudamat.euclid_norm.restype = ct.c_float
_cudamat.selectRows.restype = ct.c_int
_cudamat.setSelectedRows.restype = ct.c_int
_cudamat.vdot.restype = ct.c_float
_cudamat.dot.restype = ct.c_int


def deprecated(func):
    """This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emmitted
    when the function is used."""

    def newFunc(*args, **kwargs):
        warnings.warn("Call to deprecated function %s." % func.__name__,
                      category=DeprecationWarning)
        return func(*args, **kwargs)
    newFunc.__name__ = func.__name__
    newFunc.__doc__ = func.__doc__
    newFunc.__dict__.update(func.__dict__)
    return newFunc


class CUDAMatException(Exception):
    pass


def get_last_cuda_error():
    return str(_cudamat.get_last_cuda_error())


def generate_exception(err_code):
    """
    Return a CUDAMatException object based on the error code err_code.
    """

    if err_code == -1:
        return CUDAMatException("Incompatible matrix dimensions.")
    elif err_code == -2:
        return CUDAMatException("CUBLAS error.")
    elif err_code == -3:
        return CUDAMatException("CUDA error: " + get_last_cuda_error())
    elif err_code == -4:
        return CUDAMatException("Operation not supported on views.")
    elif err_code == -5:
        return CUDAMatException(
            "Operation not supported on transposed matrices.")
    elif err_code == -6:
        return CUDAMatException("")
    elif err_code == -7:
        return CUDAMatException("Incompatible transposedness.")
    elif err_code == -8:
        return CUDAMatException("Matrix is not in device memory.")
    elif err_code == -9:
        return CUDAMatException("Operation not supported.")


class cudamat(ct.Structure):
    _fields_ = [('data_host', ct.POINTER(ct.c_float)),
                ('data_device', ct.POINTER(ct.c_float)),
                ('on_device', ct.c_int),
                ('on_host', ct.c_int),
                ('size', ct.c_int * 2),
                ('is_trans', ct.c_int),
                ('owns_data', ct.c_int)]


class rnd_struct(ct.Structure):
    _fields_ = [('dev_rnd_mults', ct.POINTER(ct.c_uint)),
                ('dev_rnd_words', ct.POINTER(ct.c_longlong))]


class TransposedCUDAMatrix(object):
    def __init__(self, mat):
        self.mat = cudamat()
        ct.memmove(ct.pointer(self.mat), ct.pointer(mat), ct.sizeof(self.mat))
        self.mat.is_trans = 1
        self.p_mat = ct.pointer(self.mat)


class CUDAMatrix(object):
    """
    A CUDAMatrix object represents a matrix of single precision floating point
    numbers on a GPU.
    """

    def overwrite(self, array, copy_to_device=True):
        """Overwrites self with array.

        'array' should have a size smaller than that of the array used to
        initialize the CUDAMatrix. The method will not throw an Exception just
        yet if this is not true. It will throw exceptions or behave in strange
        ways later on.
        """
        assert type(array) == np.ndarray, 'array must be a np.ndarray.'
        array = reformat(array)
        self.numpy_array = array
        _cudamat.init_from_array(
            self.p_mat, array.ctypes.data_as(ct.POINTER(ct.c_float)),
            ct.c_int(array.shape[0]), ct.c_int(array.shape[1]))
        _cudamat.set_on_device(self.p_mat)
        if copy_to_device:
            err_code = _cudamat.copy_to_device(self.p_mat)
            if err_code:
                raise generate_exception(err_code)

    def __init__(self, array, copy_to_device=True):
        """
        Initializes a new matrix object in one of two ways. If array is a numpy
        ndarray, memory for a matrix with the same dimensions is allocated on
        the GPU. If the copy_to_device flag is set to True, the GPU matrix is
        initialized with the given ndarray. If array is not an ndarray, it must
        be a cudamat structure (typically the user will never use this way of
        calling __init__).
        """

        if type(array) == np.ndarray:
            # Convert array to float32 in FORTRAN order
            array = reformat(array)

            # Initialize as a ndarray-tied matrix.
            self.mat = cudamat()
            self.size = self.mat.size
            self.p_mat = ct.pointer(self.mat)
            self.numpy_array = array
            _cudamat.init_from_array(
                self.p_mat, array.ctypes.data_as(ct.POINTER(ct.c_float)),
                ct.c_int(array.shape[0]), ct.c_int(array.shape[1]))
            if copy_to_device:
                err_code = _cudamat.copy_to_device(self.p_mat)
                if err_code:
                    raise generate_exception(err_code)
        else:
            # Initialize based on existing cudamat structure.
            mat = array
            self.mat = mat
            self.p_mat = ct.pointer(self.mat)
        self.T = TransposedCUDAMatrix(self.mat)
        # Keep a reference to free device memory in case of a crash.
        self.__free_device_memory = _cudamat.free_device_memory

    @staticmethod
    def init_random(seed=0):
        """ Initialize and seed the random number generator."""

        CUDAMatrix.rndInitialized = 1
        CUDAMatrix.rnd_state = rnd_struct()
        CUDAMatrix.rnd_state_p = ct.pointer(CUDAMatrix.rnd_state)
        cudamat_path = os.path.join(
            os.path.abspath(os.path.dirname(__file__)),
            'rnd_multipliers_32bit.txt')
        err_code = _cudamat.init_random(CUDAMatrix.rnd_state_p, ct.c_int(seed),
                                        cudamat_path)
        if err_code:
            raise generate_exception(err_code)

    @property
    def shape(self):
        return (self.mat.size[0], self.mat.size[1])

    def set_shape(self, shape):
        """
        Sets the shape of the array to the given array.
        Highly unsafe method. Does no checking.
        Do not use this unless you know what you are doing.
        """

        m = ct.c_uint(shape[0])
        n = ct.c_uint(shape[1])
        err_code = _cudamat.set_shape(self.p_mat, m, n)
        if err_code:
            raise generate_exception(err_code)
        return self

    def reshape(self, shape):
        """
        Reshapes self to have the given shape. The number of elements cannot
        change as this only changes how the contents are interpreted.
        """

        m, n = shape
        mlen = self.shape[0] * self.shape[1]
        if m == -1:
            assert n > 0 and mlen % n == 0
            m = mlen / n
        elif n == -1:
            assert m > 0 and mlen % m == 0
            n = mlen / m
        err_code = _cudamat.reshape(self.p_mat, ct.c_uint(m), ct.c_uint(n))
        if err_code:
            raise generate_exception(err_code)
        return self

    def blockify(source, blocksize, target=None):
        if target is None:
            target = source
        err_code = _cudamat.blockify(source.p_mat, target.p_mat,
                                     ct.c_uint(blocksize))
        if err_code:
            raise generate_exception(err_code)
        return target

    def generate_translations(source, source_w, target_w, off_x, off_y,
                              target=None):
        num_channels = source.shape[0] / (source_w**2)
        if target is None:
            batch_s = source.shape[1]
            target = empty((target_w**2, batch_s))

        err_code = _cudamat.generate_translations_big_var_off(
            source.p_mat, target.p_mat, off_x.p_mat, off_y.p_mat,
            ct.c_uint(source_w), ct.c_uint(target_w), ct.c_uint(num_channels))

        if err_code:
            raise generate_exception(err_code)
        return target

    def asarray(self):
        """Copy the matrix to an ndarray on the CPU and return it."""

        self.copy_to_host()
        return self.numpy_array

    def copy_to_device(self):
        """Copy the matrix to the GPU."""

        err_code = _cudamat.copy_to_device(self.p_mat)
        if err_code:
            raise generate_exception(err_code)

    def copy_to_host(self):
        """Copy the matrix to the CPU."""

        if not self.mat.on_host:
            # allocate host storage if necessary
            m = self.mat.size[0]
            n = self.mat.size[1]
            self.numpy_array = np.empty((m, n), dtype=np.float32, order="F")
            self.mat.data_host = \
                self.numpy_array.ctypes.data_as(ct.POINTER(ct.c_float))
            self.mat.on_host = 1

        err_code = _cudamat.copy_to_host(self.p_mat)
        if err_code:
            raise generate_exception(err_code)

    def assign(self, val):
        """Assign val to self, where val can be a scalar or a CUDAMatrix
        with the same dimensions as self. """

        if isinstance(val, CUDAMatrix):
            err_code = _cudamat.copy_on_device(val.p_mat, self.p_mat)
        elif isinstance(val, (int, float)):
            err_code = _cudamat.assign_scalar(self.p_mat, ct.c_float(val))
        else:
            raise ValueError(
                "Assigned value must be of type CUDAMatrix, int or float.")
        if err_code:
            raise generate_exception(err_code)
        return self

    def free_device_memory(self):
        """Free memory used up by the matrix on the GPU."""

        err_code = _cudamat.free_device_memory(self.p_mat)
        if err_code:
            raise generate_exception(err_code)

    def set_trans(self, is_trans):
        """Set the transposedness flag to is_trans."""

        _cudamat.set_transpose(self.p_mat, ct.c_int(1 * is_trans))

    def slice(self, first_col, last_col):
        mat = cudamat()

        if self.mat.size[0] == 1 or self.mat.size[1] == 1:
            err_code = _cudamat.get_vector_slice(
                self.p_mat, ct.pointer(mat), ct.c_int(first_col),
                ct.c_int(last_col))
        else:
            err_code = _cudamat.get_slice(
                self.p_mat, ct.pointer(mat), ct.c_int(first_col),
                ct.c_int(last_col))

        if err_code:
            raise generate_exception(err_code)

        new_mat = CUDAMatrix(mat)

        try:
            new_mat.sliceof = self.sliceof
        except:
            new_mat.sliceof = self

        return new_mat

    def get_col_slice(self, first_col, last_col, target=None):
        col_slice = self.slice(first_col, last_col)

        if target:
            target.assign(col_slice)
            return target
        else:
            return col_slice

    def set_col_slice(self, first_col, last_col, mat):
        self.slice(first_col, last_col).assign(mat)

        return self

    def get_row_slice(self, start, end, target=None):
        """
        Get the rows with indices start through end. If target is not provided
        memory for a new matrix will be allocated.
        """

        width = self.shape[1]

        if not target:
            target = empty((end-start, width))

        err_code = _cudamat.get_row_slice(
            self.p_mat, target.p_mat, ct.c_int(start), ct.c_int(end))
        if err_code:
            raise generate_exception(err_code)

        return target

    def set_row_slice(self, start, end, mat):
        """
        Assign the contents of mat to the rows with indices start through end.
        """

        err_code = _cudamat.set_row_slice(
            mat.p_mat, self.p_mat, ct.c_int(start), ct.c_int(end))
        if err_code:
            raise generate_exception(err_code)

        return self

    def transpose(self, target=None):
        """Return a transposed copy of the matrix."""
        if not target:
            target = empty((self.shape[1], self.shape[0]))

        err_code = _cudamat.copy_transpose(self.p_mat, target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return target

    def fill_with_rand(self):
        """
        Fill matrix on the GPU with random numbers drawn from the uniform
        distribution over the (0,1) interval.
        """

        err_code = _cudamat.fill_with_rand(CUDAMatrix.rnd_state_p, self.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return self

    def fill_with_randn(self):
        """
        Fill matrix on the GPU with random numbers drawn from the standard
        normal distribution.
        """

        err_code = _cudamat.fill_with_randn(CUDAMatrix.rnd_state_p, self.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return self

    def dropout(self, dropprob, val=0.0):
        """
        Drop entries in this matrix uniformly randomly with given probability
        and set the dropped out unit to state val.
        """
        err_code = _cudamat.dropout(CUDAMatrix.rnd_state_p, self.p_mat,
                                    ct.c_float(dropprob), ct.c_float(val))
        if err_code:
            raise generate_exception(err_code)

        return self

    def sample_bernoulli(self, target=None):
        """
        Sample a bernoulli distribution. Choose 1 with probability given by
        entries of self, 0 otherwise.
        """

        if not target:
            target = self
        err_code = _cudamat.sample_bernoulli(
            CUDAMatrix.rnd_state_p, self.p_mat, target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return self

    def sample_bernoulli_tanh(self, target=None):
        """
        Sample a bernoulli distribution. Choose 1 with probability given by
        entries of (1+self)/2, -1 otherwise.
        """

        if not target:
            target = self
        err_code = _cudamat.sample_bernoulli_tanh(
            CUDAMatrix.rnd_state_p, self.p_mat, target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return self

    def sample_poisson(self, target=None):
        """
        Sample a poisson distribution. Choose 1 with probability given by
        entries of self. Not implemented yet.
        """

        if not target:
            target = self
        err_code = _cudamat.sample_poisson(
            CUDAMatrix.rnd_state_p, self.p_mat, target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return self

    def sample_gaussian(self, mult=1.0, target=None):
        """Add zero mean gaussian noise to the matrix. mult is the stddev."""

        if not target:
            target = self
        err_code = _cudamat.sample_gaussian(
            CUDAMatrix.rnd_state_p, self.p_mat, target.p_mat, ct.c_float(mult))
        if err_code:
            raise generate_exception(err_code)

        return self

    def perturb_energy_for_softmax_sampling(self, target=None):
        """Add by -log(-log(rand))."""

        if not target:
            target = self
        err_code = _cudamat.perturb_energy(
            CUDAMatrix.rnd_state_p, self.p_mat, target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return self

    def perturb_prob_for_softmax_sampling(self, target=None):
        """Divide by -log(rand)."""

        if not target:
            target = self
        err_code = _cudamat.perturb_prob(
            CUDAMatrix.rnd_state_p, self.p_mat, target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return self

    def add_col_vec(self, vec, target=None):
        """
        Add vector vec to every column of the matrix. If a target is provided,
        it is used to store the result instead of self.
        """

        if not target:
            target = self

        err_code = _cudamat.add_col_vec(self.p_mat, vec.p_mat, target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return target

    def add_col_mult(self, vec, mult, target=None):
        """
        Add a multiple of vector vec to every column of the matrix. If a target
        is provided, it is used to store the result instead of self.
        """

        if not target:
            target = self

        err_code = _cudamat.add_col_mult(
            self.p_mat, vec.p_mat, target.p_mat, ct.c_float(mult))
        if err_code:
            raise generate_exception(err_code)

        return target

    def mult_diagonal(self, val, target=None):
        """
        Mult val to the diagonal of self. If a target
        is provided, it is used to store the result instead of self.
        """

        if not target:
            target = self

        assert self.shape[0] == self.shape[1], 'self must be a square matrix'
        if isinstance(val, CUDAMatrix):
            err_code = \
                _cudamat.mult_diagonal(self.p_mat, val.p_mat, target.p_mat)
        elif isinstance(val, (int, float)):
            err_code = _cudamat.mult_diagonal_scalar(
                self.p_mat, ct.c_float(val), target.p_mat)
        else:
            raise ValueError("Value must be of type CUDAMatrix, int or float.")

        if err_code:
            raise generate_exception(err_code)

        return target

    def add_diagonal(self, val, target=None):
        """
        Add val to the diagonal of self. If a target
        is provided, it is used to store the result instead of self.
        """

        if not target:
            target = self

        assert self.shape[0] == self.shape[1], 'self must be a square matrix'
        if isinstance(val, CUDAMatrix):
            err_code = \
                _cudamat.add_diagonal(self.p_mat, val.p_mat, target.p_mat)
        elif isinstance(val, (int, float)):
            err_code = _cudamat.add_diagonal_scalar(
                self.p_mat, ct.c_float(val), target.p_mat)
        else:
            raise ValueError("Value must be of type CUDAMatrix, int or float.")

        if err_code:
            raise generate_exception(err_code)

        return target

    def add_row_mult(self, vec, mult, target=None):
        """
        Add a multiple of vector vec to every row of the matrix. If a target
        is provided, it is used to store the result instead of self.
        """

        if not target:
            target = self

        err_code = _cudamat.add_row_mult(
            self.p_mat, vec.p_mat, target.p_mat, ct.c_float(mult))
        if err_code:
            raise generate_exception(err_code)

        return target

    def add_row_vec(self, vec, target=None):
        """
        Add vector vec to every row of the matrix. If a target is provided,
        it is used to store the result instead of self.
        """

        if not target:
            target = self

        err_code = _cudamat.add_row_vec(self.p_mat, vec.p_mat, target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return target

    def mult_by_col(self, vec, target=None):
        """
        Multiply vector vec into every column of the matrix. If a target is
        provided, it is used to store the result instead of self.
        """

        if not target:
            target = self

        err_code = \
            _cudamat.mult_by_col_vec(self.p_mat, vec.p_mat, target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return target

    def mult_by_row(self, vec, target=None):
        """
        Multiply vector vec into every row of the matrix. If a target is
        provided, it is used to store the result instead of self.
        """

        if not target:
            target = self

        err_code = \
            _cudamat.mult_by_row_vec(self.p_mat, vec.p_mat, target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return target

    def div_by_col(self, vec, target=None):
        """
        Multiply vector vec into every column of the matrix. If a target is
        provided, it is used to store the result instead of self.
        """

        if not target:
            target = self

        err_code = _cudamat.div_by_col_vec(self.p_mat, vec.p_mat, target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return target

    def div_by_row(self, vec, target=None):
        """
        Divide vector vec into every row of the matrix. If a target is
        provided, it is used to store the result instead of self.
        """

        if not target:
            target = self

        err_code = _cudamat.div_by_row_vec(self.p_mat, vec.p_mat, target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return target

    def sum(self, axis=None, target=None):
        """
        Sum the matrix along the given dimension, where 0 represents the
        leading dimension and 1 represents the non-leading dimension.
        If None, the sum of all elements is returned.
        If a target is not provided, a new vector is created for storing
        the result.
        """

        if axis is None:
            return vdot(
                self, CUDAMatrix.ones.slice(0, self.shape[0] * self.shape[1]))
        else:
            return sum(self, axis, target)

    def add_sums(self, mat, axis, mult=1.):
        """
        Add a multiple of the sums of the matrix mat along the given dimension
        to self.
        """

        m = _cudamat.get_leading_dimension(mat.p_mat)
        n = _cudamat.get_nonleading_dimension(mat.p_mat)

        if axis == 0:
            # sum along leading dimension
            left = CUDAMatrix.ones.slice(0, m)
            left.set_trans(True)
            right = mat
        elif axis == 1:
            # sum along non-leading dimension
            left = mat
            right = CUDAMatrix.ones.slice(0, n)

        err_code = _cudamat.dot(left.p_mat, right.p_mat, self.p_mat,
                                ct.c_float(1.), ct.c_float(mult))
        if err_code:
            raise generate_exception(err_code)

        return self

    def less_than_eq(self, val, target=None):
        """
        Perform the operation target = 1. * (self < val), where val can be a
        matrix or a scalar.
        """

        if not target:
            target = self

        if isinstance(val, (int, float)):
            err_code = _cudamat.less_than_eq_scalar(
                self.p_mat, ct.c_float(val), target.p_mat)
        else:
            err_code = _cudamat.less_than_eq(
                self.p_mat, val.p_mat, target.p_mat)

        if err_code:
            raise generate_exception(err_code)

        return target

    def less_than(self, val, target=None):
        """
        Perform the operation target = 1. * (self < val), where val can be a
        matrix or a scalar.
        """

        if not target:
            target = self

        if isinstance(val, (int, float)):
            err_code = _cudamat.less_than_scalar(
                self.p_mat, ct.c_float(val), target.p_mat)
        else:
            err_code = _cudamat.less_than(self.p_mat, val.p_mat, target.p_mat)

        if err_code:
            raise generate_exception(err_code)

        return target

    def greater_than_eq(self, val, target=None):
        """
        Perform the operation target = 1. * (self > val), where val can be a
        matrix or a scalar.
        """

        if not target:
            target = self

        if isinstance(val, (int, float)):
            err_code = _cudamat.greater_than_eq_scalar(
                self.p_mat, ct.c_float(val), target.p_mat)
        else:
            err_code = _cudamat.greater_than_eq(
                self.p_mat, val.p_mat, target.p_mat)

        if err_code:
            raise generate_exception(err_code)

        return target

    def greater_than(self, val, target=None):
        """
        Perform the operation target = 1. * (self > val), where val can be a
        matrix or a scalar.
        """

        if not target:
            target = self

        if isinstance(val, (int, float)):
            err_code = _cudamat.greater_than_scalar(
                self.p_mat, ct.c_float(val), target.p_mat)
        else:
            err_code = \
                _cudamat.greater_than(self.p_mat, val.p_mat, target.p_mat)

        if err_code:
            raise generate_exception(err_code)

        return target

    def upper_bound(self, val, target=None):
        """
        Perform the operation target = (self > val) ? val:self, where val can
        be a matrix or a scalar.
        """
        if not target:
            target = self

        if isinstance(val, (int, float)):
            err_code = _cudamat.upper_bound_scalar(
                self.p_mat, ct.c_float(val), target.p_mat)
        else:
            err_code = \
                _cudamat.upper_bound(self.p_mat, val.p_mat, target.p_mat)

        if err_code:
            raise generate_exception(err_code)

        return target

    def lower_bound(self, val, target=None):
        """
        Perform the operation target = (self < val) ? val:self, where val can
        be a matrix or a scalar.
        """
        if not target:
            target = self

        if isinstance(val, (int, float)):
            err_code = _cudamat.lower_bound_scalar(
                self.p_mat, ct.c_float(val), target.p_mat)
        else:
            err_code = \
                _cudamat.lower_bound(self.p_mat, val.p_mat, target.p_mat)

        if err_code:
            raise generate_exception(err_code)

        return target

    def cumsum(self, axis, temp=None, target=None):
        """Cumulative sum along axis."""

        m, n = self.shape
        assert axis == 0, 'axis = 1 not implemented.'
        if not target:
            target = empty((m, n))
        if not temp:
            temp = empty((m, n))
        """
        elif axis == 1:
            if not target:
                target = empty((m, 1))
        """
        err_code = _cudamat.cumsum_by_axis(
            self.p_mat, target.p_mat, temp.p_mat, ct.c_int(axis))

        if err_code:
            raise generate_exception(err_code)

        return target

    def choose_max_and_accumulate(self, acc):
        """
        Find the maximum value along the given dimension, where 0 represents
        the leading dimension and 1 represents the non-leading dimension.
        If a target is not prvided, a new vector is created for storing
        the result.
        """

        m, n = self.shape

        err_code = _cudamat.choose_max_and_accumulate(self.p_mat, acc.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return acc

    def choose_max(self, axis, target=None):
        """Set the argmax along axis to 1 and rest to zero."""

        m, n = self.shape
        assert axis == 0, 'Axis = 1 not implemented.'
        if not target:
            target = self

        err_code = _cudamat.choose_max_by_axis(
            self.p_mat, target.p_mat, ct.c_int(axis))
        if err_code:
            raise generate_exception(err_code)

        return target

    def max(self, axis, target=None):
        """
        Find the maximum value along the given dimension, where 0 represents
        the leading dimension and 1 represents the non-leading dimension.
        If a target is not prvided, a new vector is created for storing
        the result.
        """

        m, n = self.shape
        if axis == 0:
            if not target:
                target = empty((1, n))
        elif axis == 1:
            if not target:
                target = empty((m, 1))

        err_code = \
            _cudamat.max_by_axis(self.p_mat, target.p_mat, ct.c_int(axis))
        if err_code:
            raise generate_exception(err_code)

        return target

    def argmax(self, axis, target=None):
        """
        Find the index with the maximum value along the given dimension,
        where 0 represents the leading dimension and 1 represents the
        non-leading dimension. If a target is not prvided, a new vector is
        created for storing the result.
        """

        m, n = self.shape
        if axis == 0:
            if not target:
                target = empty((1, n))
        elif axis == 1:
            if not target:
                target = empty((m, 1))
        err_code = _cudamat.argmax_by_axis(
            self.p_mat, target.p_mat, ct.c_int(axis))

        if err_code:
            raise generate_exception(err_code)

        return target

    def add_sqsums(self, mat, axis, mult=1.):
        """
        Add the sum of squares of mat along the given dimension to self.
        0 represents the leading dimension and 1 represents the non-leading
        dimension.
        """

        (m, n) = mat.shape
        if axis == 0:
            assert self.shape == (1, n), \
                'Self has shape %s but mat has shape %s' % \
                (self.shape, mat.shape)
        elif axis == 1:
            assert self.shape == (m, 1)

        err_code = _cudamat.sqsum_by_axis(
            mat.p_mat, self.p_mat, ct.c_int(axis), ct.c_float(mult),
            ct.c_float(1.0))
        if err_code:
            raise generate_exception(err_code)

    def sqsum(self, axis, target=None):
        """
        Find the sum of squares along the given dimension, where 0 represents
        the leading dimension and 1 represents the non-leading dimension.
        If a target is not prvided, a new vector is created for storing the
        result.
        """

        m, n = self.shape
        if axis == 0:
            if not target:
                target = empty((1, n))
        elif axis == 1:
            if not target:
                target = empty((m, 1))

        err_code = _cudamat.sqsum_by_axis(
            self.p_mat, target.p_mat, ct.c_int(axis), 1.0, 0.0)
        if err_code:
            raise generate_exception(err_code)

        return target

    def norm_limit(self, norm, axis, target=None):
        """
        Limit the norm along the given dimension to be 'norm', where 0
        represents the leading dimension and 1 represents the non-leading
        dimension. If a target is not provided, self is used as target.
        """

        m, n = self.shape
        if not target:
            target = self
        err_code = _cudamat.normlimit_by_axis(
            self.p_mat, target.p_mat, ct.c_int(axis), ct.c_float(norm))
        if err_code:
            raise generate_exception(err_code)

        return target

    def apply_softmax(self, target=None):
        """Apply the softmax activation function."""

        return softmax(self, target)

    def sign(self, target=None):
        """Find the sign of each element of the matrix."""

        if not target:
            target = empty((self.mat.size[0], self.mat.size[1]))

        err_code = _cudamat.sign(self.p_mat, target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return target

    def apply_cos(self, target=None):
        """Apply the cos sigmoid to each element of the matrix."""

        return cos(self, target)

    def apply_sin(self, target=None):
        """Apply the sin sigmoid to each element of the matrix."""

        return sin(self, target)

    def apply_sigmoid(self, target=None):
        """Apply the logistic sigmoid to each element of the matrix."""

        return sigmoid(self, target)

    def reciprocal(self, target=None):
        """Find the reciprocal of each element of the matrix."""

        if not target:
            target = self

        err_code = _cudamat.reciprocal(self.p_mat, target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return target

    def dot(self, mat2, mult=1.0, target=None):
        """
        Multiply the matrix by mat2 from the right and multiply by scalar mult.
        """

        return dot(self, mat2, mult, target)

    def add_dot(self, m1, m2, mult=1.0):
        """Add the dot product of m1 and m2 to the matrix."""

        err_code = _cudamat.dot(
            m1.p_mat, m2.p_mat, self.p_mat, ct.c_float(1.), ct.c_float(mult))
        if err_code:
            raise generate_exception(err_code)

        return self

    def subtract_dot(self, m1, m2):
        """Subtract the dot product of m1 and m2 from the matrix."""

        err_code = _cudamat.dot(
            m1.p_mat, m2.p_mat, self.p_mat, ct.c_float(1.), ct.c_float(-1.))
        if err_code:
            raise generate_exception(err_code)

        return self

    def add_mult_sign(self, mat2, mult=1.):
        """Add multiple of sign of mat2 to the matrix."""

        err_code = \
            _cudamat.add_mult_sign(self.p_mat, mat2.p_mat, ct.c_float(mult))
        if err_code:
            raise generate_exception(err_code)

        return self

    def add_mult(self, mat2, mult=1.):
        """Add multiple of mat2 to the matrix."""

        err_code = _cudamat.add_mult(self.p_mat, mat2.p_mat, ct.c_float(mult))
        if err_code:
            raise generate_exception(err_code)

        return self

    def subtract_mult(self, mat2, mult=1.):
        """Subtract a multiple of mat2 from the matrix."""

        err_code = \
            _cudamat.add_mult(self.p_mat, mat2.p_mat, ct.c_float(-1. * mult))
        if err_code:
            raise generate_exception(err_code)

        return self

    def add(self, val, target=None):
        """
        Add val to self, where val can be a scalar or a CUDAMatrix with the
        same dimensions as self.
        """

        if not target:
            target = self

        if isinstance(val, CUDAMatrix):
            err_code = _cudamat.add_elementwise(
                self.p_mat, val.p_mat, target.p_mat)
        elif isinstance(val, (int, float)):
            err_code = _cudamat.add_scalar(
                self.p_mat, ct.c_float(val), target.p_mat)
        else:
            raise ValueError("Value must be of type CUDAMatrix, int or float.")

        if err_code:
            raise generate_exception(err_code)
        return target

    def accumulate_columns(self, indices, target, mult=1.0, avg=False):
        if not target:
            target = self
        if avg:
            avgg = 1
        else:
            avgg = 0
        err_code = _cudamat.accumulate_columns(
            self.p_mat, indices.p_mat, target.p_mat, ct.c_float(mult),
            ct.c_int(avgg))
        if err_code:
            raise generate_exception(err_code)
        return target

    def expand(self, expansion_indices, target):

        err_code = _cudamat.expand(
            self.p_mat, expansion_indices.p_mat, target.p_mat)

        if err_code:
            raise generate_exception(err_code)

        return target

    def expand_and_add(self, val, expansion_indices, target=None, mult=1.0):

        if not target:
            target = self

        if isinstance(val, CUDAMatrix) and \
                isinstance(expansion_indices, CUDAMatrix):
            err_code = _cudamat.expand_and_add(
                self.p_mat, val.p_mat, expansion_indices.p_mat, target.p_mat,
                ct.c_float(mult))
        else:
            raise ValueError("Value must be of type CUDAMatrix, int or float.")

        if err_code:
            raise generate_exception(err_code)

        return target

    def subtract(self, val, target=None):
        """Subtract val from self, where val can be a scalar or a CUDAMatrix
        with the same dimensions as self. """

        if not target:
            target = self
        if isinstance(val, CUDAMatrix):
            err_code = _cudamat.subtract_elementwise(
                self.p_mat, val.p_mat, target.p_mat)
        elif isinstance(val, (int, float)):
            err_code = _cudamat.add_scalar(
                self.p_mat, ct.c_float(-1*val), target.p_mat)
        else:
            raise ValueError("Value must be of type CUDAMatrix, int or float.")
        if err_code:
            raise generate_exception(err_code)
        return target

    def divide(self, val, target=None):
        """Divide self by val, where val can be a scalar or a CUDAMatrix with
        the same dimensions as self. """

        if not target:
            target = self
        if isinstance(val, CUDAMatrix):
            err_code = _cudamat.divide_elementwise(
                self.p_mat, val.p_mat, target.p_mat)
        elif isinstance(val, (int, float)):
            err_code = _cudamat.divide_by_scalar(
                self.p_mat, ct.c_float(val), target.p_mat)
        else:
            raise ValueError("Value must be of type CUDAMatrix, int or float.")
        if err_code:
            raise generate_exception(err_code)
        return target

    def mult(self, val, target=None):
        """
        Multiply self by val, where val can be a scalar or a CUDAMatrix with
        the same dimensions as self.
        """

        if not target:
            target = self

        if isinstance(val, CUDAMatrix):
            err_code = \
                _cudamat.mult_elementwise(self.p_mat, val.p_mat, target.p_mat)
        elif isinstance(val, (int, float)):
            err_code = _cudamat.mult_by_scalar(
                self.p_mat, ct.c_float(val), target.p_mat)
        else:
            raise ValueError("Value must be of type CUDAMatrix, int or float.")
        if err_code:
            raise generate_exception(err_code)
        return target

    def apply_cos_deriv(self, val, target=None):
        """Apply cos derivative, where val is the activation of cos units."""

        if not target:
            target = self
        if isinstance(val, CUDAMatrix):
            err_code = \
                _cudamat.apply_cos_deriv(self.p_mat, val.p_mat, target.p_mat)
        else:
            raise ValueError("Value must be of type CUDAMatrix.")

        if err_code:
            raise generate_exception(err_code)

        return target

    def apply_sin_deriv(self, val, target=None):
        """Apply sin derivative, where val is the activation of sin units."""
        if not target:
            target = self
        if isinstance(val, CUDAMatrix):
            err_code = \
                _cudamat.apply_sin_deriv(self.p_mat, val.p_mat, target.p_mat)
        else:
            raise ValueError("Value must be of type CUDAMatrix.")
        if err_code:
            raise generate_exception(err_code)
        return target

    def get_softmax_correct(self, labels, target):
        """target[i] = 1, iff labels[i] is correctly predicted; 0 otherwise."""
        assert labels.shape == (1, self.shape[1])
        assert target.shape == labels.shape
        if isinstance(labels, CUDAMatrix):
            err_code = _cudamat.get_softmax_correct(
                self.p_mat, labels.p_mat, target.p_mat)
        else:
            raise ValueError("labels must be of type CUDAMatrix.")
        if err_code:
            raise generate_exception(err_code)
        return target

    def get_softmax_cross_entropy(self, labels, target, tiny=1e-10):
        """target[i] = -log(self[label[i]] + tiny)."""

        assert labels.shape == (1, self.shape[1])
        assert target.shape == labels.shape
        if isinstance(labels, CUDAMatrix):
            err_code = _cudamat.get_softmax_cross_entropy(
                self.p_mat, labels.p_mat, target.p_mat, ct.c_float(tiny))
        else:
            raise ValueError("labels must be of type CUDAMatrix.")
        if err_code:
            raise generate_exception(err_code)
        return target

    def apply_softmax_grad(self, labels, target=None):
        """Apply softmax derivative, where labels are the correct labels."""

        if not target:
            target = self
        assert target.shape == self.shape
        if isinstance(labels, CUDAMatrix):
            if labels.shape == (1, self.shape[1]):
                err_code = _cudamat.apply_softmax_grad(
                    self.p_mat, labels.p_mat, target.p_mat)
            elif labels.shape == self.shape:
                err_code = _cudamat.subtract_elementwise(
                    self.p_mat, labels.p_mat, target.p_mat)
            else:
                raise ValueError("wrong shape of labels matrix.")
        else:
            raise ValueError("labels must be of type CUDAMatrix.")
        if err_code:
            raise generate_exception(err_code)
        return target

    def apply_logistic_deriv(self, val, target=None):
        """
        Apply logistic derivative, where val is the logistic unit activations.
        """

        if not target:
            target = self
        if isinstance(val, CUDAMatrix):
            err_code = _cudamat.apply_logistic_deriv(
                self.p_mat, val.p_mat, target.p_mat)
        else:
            raise ValueError("Value must be of type CUDAMatrix.")
        if err_code:
            raise generate_exception(err_code)
        return target

    def apply_tanh_deriv(self, val, target=None):
        """Apply tanh derivative, where val is the activation of the units."""

        if not target:
            target = self

        if isinstance(val, CUDAMatrix):
            err_code = _cudamat.apply_tanh_deriv(
                self.p_mat, val.p_mat, target.p_mat)
        else:
            raise ValueError("Value must be of type CUDAMatrix.")

        if err_code:
            raise generate_exception(err_code)

        return target

    def apply_rectified_linear_deriv(self, val, target=None):
        """
        Apply rectified linear derivative, where val is the activation of the
        units.
        """

        if not target:
            target = self

        if isinstance(val, CUDAMatrix):
            err_code = _cudamat.apply_rectified_linear_deriv(
                self.p_mat, val.p_mat, target.p_mat)
        else:
            raise ValueError("Value must be of type CUDAMatrix.")

        if err_code:
            raise generate_exception(err_code)

        return target

    def apply_rectified_linear_smooth_deriv(self, val, target=None):
        """
        Apply rectified linear smooth derivative, where val is the activation
        of the units.
        """

        if not target:
            target = self

        if isinstance(val, CUDAMatrix):
            err_code = _cudamat.apply_rectified_linear_smooth_deriv(
                self.p_mat, val.p_mat, target.p_mat)
        else:
            raise ValueError("Value must be of type CUDAMatrix.")

        if err_code:
            raise generate_exception(err_code)

        return target

    @deprecated
    def assign_scalar(self, alpha):
        """Assign scalar alpha to every element of the matrix."""

        err_code = _cudamat.assign_scalar(self.p_mat, ct.c_float(alpha))
        if err_code:
            raise generate_exception(err_code)

        return self

    @deprecated
    def mult_by_scalar(self, alpha, target=None):
        """Multiply the matrix by a scalar."""

        if not target:
            target = self

        err_code = _cudamat.mult_by_scalar(
            self.p_mat, ct.c_float(alpha), target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return target

    @deprecated
    def div_by_scalar(self, alpha, target=None):
        """Divide the matrix by a scalar."""

        if not target:
            target = self

        err_code = _cudamat.divide_by_scalar(
            self.p_mat, ct.c_float(alpha), target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return target

    @deprecated
    def add_scalar(self, alpha, target=None):
        """Increment the matrix by a scalar."""

        if not target:
            target = self

        err_code = _cudamat.add_scalar(
            self.p_mat, ct.c_float(alpha), target.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return target

    def euclid_norm(self):
        err_code = ct.c_int(0)
        res = _cudamat.euclid_norm(self.p_mat, ct.byref(err_code))

        if err_code:
            raise generate_exception(err_code.value)

        return res

    def select_columns(self, indices, target):
        """
        copies some columns of self into target.
        <indices> must be a row vector. Its elements are float32's representing
        integers, e.g. "34.0" means the integer "34".
        after this call, for all r,c, target[r,c]=self[r,indices[c]].
        This returns target.
        Negative indices are interpreted in the usual Python way: all elements
        of <indices> had better be in the range
        [-self.shape[1], self.shape[1]-1].
        This does bounds checking, but out of bounds indices do not raise an
        exception (because the programmer was lazy).
        Instead, they result in NaN values in <target>.
        """

        err_code = _cudamat.selectRows(self.p_mat, target.p_mat, indices.p_mat)

        if err_code:
            raise generate_exception(err_code)

        return target

    def swap_columns(self, indices1, indices2, target):
        """
        swap columns at indices1 of self with columns at indices2 of target.
        <indices1> and <indices2> must be row vectors of equal length.
        Its elements are float32's representing integers, e.g. "34.0" means
        the integer "34".
        after this call, for all r,c, target[r,indices2[c]=self[r,indices1[c]].
        self can be same as target, but then the result will be
        non-deterministic if there is overlap between indices1 and indices2.
        Can be used for in-place shuffling by making sure indices1 and indices2
        do not overlap.
        This returns target.
        Negative indices are interpreted in the usual Python way: all elements
        of <indices> had better be in the range
        [-self.shape[1], self.shape[1]-1].
        This does bounds checking, but out of bounds indices do not raise an
        exception (because the programmer was lazy).
        Instead, they result in NaN values in <target>.
        """

        assert indices1.shape == indices2.shape
        err_code = _cudamat.swapColumns(
            self.p_mat, target.p_mat, indices1.p_mat, indices2.p_mat)

        if err_code:
            raise generate_exception(err_code)

        return target

    def set_selected_columns(self, indices, source):
        """
        copies all columns of source into some columns of self.
        <indices> must be a row vector. Its elements are float32's representing
        integers, e.g. "34.0" means the integer "34". after this call, for all
        r,c, self[r,indices[c]]=source[r,c]. This returns self.
        Negative indices are interpreted in the usual Python way: all elements
        of <indices> had better be in the range
        [-self.shape[1], self.shape[1]-1].
        This does bounds checking, but out of bounds indices do not raise an
        exception (because the programmer was lazy). Instead, they result in
        NaN values in <self>.
        """

        err_code = _cudamat.setSelectedRows(
            self.p_mat, source.p_mat, indices.p_mat)
        if err_code:
            raise generate_exception(err_code)

        return self


def empty(shape):
    """Creates and returns a new CUDAMatrix with the given shape."""

    mat = cudamat()
    err_code = _cudamat.init_empty(
        ct.pointer(mat), ct.c_int(shape[0]), ct.c_int(shape[1]))

    if err_code:
        raise generate_exception(err_code)

    return CUDAMatrix(mat)


def sum(mat, axis, target=None):
    """
    Sum the matrix along the given dimension, where 0 represents the leading
    dimension and 1 represents the non-leading dimension. If a target is
    not prvided, a new vector is created for storing the result.
    """

    m = _cudamat.get_leading_dimension(mat.p_mat)
    n = _cudamat.get_nonleading_dimension(mat.p_mat)

    if axis == 0:
        # sum along leading dimension
        left = CUDAMatrix.ones.slice(0, m)
        left.set_trans(True)
        right = mat
        if not target:
            target = empty((1, n))
    elif axis == 1:
        # sum along non-leading dimension
        left = mat
        right = CUDAMatrix.ones.slice(0, n)
        if not target:
            target = empty((m, 1))

    err_code = _cudamat.dot(
        left.p_mat, right.p_mat, target.p_mat, ct.c_float(0.), ct.c_float(1.))
    if err_code:
        raise generate_exception(err_code)

    return target


def dot(m1, m2, mult=1.0, target=None):
    """Find the dot product between m1 and m2."""

    if not target:
        m = _cudamat.get_leading_dimension(m1.p_mat)
        n = _cudamat.get_nonleading_dimension(m2.p_mat)

        target = empty((m, n))

    err_code = _cudamat.dot(
        m1.p_mat, m2.p_mat, target.p_mat, ct.c_float(0.), ct.c_float(mult))
    if err_code:
        raise generate_exception(err_code)

    return target


def vdot(m1, m2):
    """Compute the vector dot product of matrices m1 and m2."""

    err_code = ct.c_int(0)
    res = _cudamat.vdot(m1.p_mat, m2.p_mat, ct.byref(err_code))

    if err_code:
        raise generate_exception(err_code.value)

    return res


def softmax(mat, target=None):
    """Apply cos to each element of the matrix mat."""

    if target:
        err_code = _cudamat.softmax(mat.p_mat, target.p_mat)
    else:
        err_code = _cudamat.softmax_overwrite(mat.p_mat)
        target = mat
    if err_code:
        raise generate_exception(err_code)
    return target


def cos(mat, target=None):
    """Apply cos to each element of the matrix mat."""

    if not target:
        target = mat

    err_code = _cudamat.apply_cos(mat.p_mat, target.p_mat)
    if err_code:
        raise generate_exception(err_code)

    return target


def sin(mat, target=None):
    """Apply sin to each element of the matrix mat."""

    if not target:
        target = mat

    err_code = _cudamat.apply_sin(mat.p_mat, target.p_mat)
    if err_code:
        raise generate_exception(err_code)

    return target


def sigmoid(mat, target=None):
    """Apply the logistic sigmoid to each element of the matrix mat."""

    if not target:
        target = mat

    err_code = _cudamat.apply_sigmoid(mat.p_mat, target.p_mat)
    if err_code:
        raise generate_exception(err_code)

    return target


def tanh(mat, target=None):
    """Apply the tanh to each element of the matrix mat."""

    if not target:
        target = mat

    err_code = _cudamat.apply_tanh(mat.p_mat, target.p_mat)
    if err_code:
        raise generate_exception(err_code)

    return target


def abs(mat, target=None):
    """Apply abs to each element of the matrix mat."""

    if not target:
        target = mat

    err_code = _cudamat.apply_abs(mat.p_mat, target.p_mat)
    if err_code:
        raise generate_exception(err_code)

    return target


def log_1_plus_exp(mat, target=None, exact=False):
    """
    Apply log(1+exp(x)) to each element of the matrix mat.
    If exact is True, use slow and accurate log and exp.
    """

    if not target:
        target = mat

    if exact:
        err_code = _cudamat.apply_log_1_plus_exp_exact(mat.p_mat, target.p_mat)
    else:
        err_code = _cudamat.apply_log_1_plus_exp(mat.p_mat, target.p_mat)
    if err_code:
        raise generate_exception(err_code)

    return target


def log(mat, tiny=0.0, target=None):
    """Find the natural logarithm of each element of the matrix mat."""

    if not target:
        target = mat

    err_code = _cudamat.apply_log(mat.p_mat, target.p_mat, ct.c_float(tiny))
    if err_code:
        raise generate_exception(err_code)

    return target


def exp(mat, target=None):
    """Apply the exponential function to each element of the matrix mat."""

    if not target:
        target = mat

    err_code = _cudamat.apply_exp(mat.p_mat, target.p_mat)
    if err_code:
        raise generate_exception(err_code)

    return target


def ceil(mat, target=None):
    """Apply the ceil function to each element of the matrix mat."""

    if not target:
        target = mat

    err_code = _cudamat.apply_ceil(mat.p_mat, target.p_mat)
    if err_code:
        raise generate_exception(err_code)

    return target


def floor(mat, target=None):
    """Apply the floor function to each element of the matrix mat."""

    if not target:
        target = mat

    err_code = _cudamat.apply_floor(mat.p_mat, target.p_mat)
    if err_code:
        raise generate_exception(err_code)

    return target


def sqrt(mat, target=None):
    """Compute the square root of each element of the matrix mat."""

    if not target:
        target = mat

    err_code = _cudamat.apply_sqrt(mat.p_mat, target.p_mat)
    if err_code:
        raise generate_exception(err_code)

    return target


def cross_entropy_bernoulli(mat, p, target=None, tiny=1e-10):
    """Compute -mat*log(p) - (1-mat).*log(1-p)"""

    if not target:
        target = mat

    if isinstance(p, CUDAMatrix):
        err_code = _cudamat.compute_cross_entropy_bernoulli(
            mat.p_mat, p.p_mat, target.p_mat, ct.c_float(tiny))
    else:
        raise ValueError("Value must be of type CUDAMatrix.")

    if err_code:
        raise generate_exception(err_code)

    return target


def cross_entropy(mat, p, target=None, tiny=1e-10):
    """Compute -mat*log(p)"""

    if not target:
        target = mat

    if isinstance(p, CUDAMatrix):
        err_code = _cudamat.compute_cross_entropy(
            mat.p_mat, p.p_mat, target.p_mat, ct.c_float(tiny))
    else:
        raise ValueError("Value must be of type CUDAMatrix.")

    if err_code:
        raise generate_exception(err_code)

    return target


def correct_preds(mat, p, target=None, cutoff=0.5):
    """Compute mat*(p >= 0.5) + (1-mat).*(p < 0.5)"""

    if not target:
        target = mat

    if isinstance(p, CUDAMatrix):
        err_code = _cudamat.correct_preds(
            mat.p_mat, p.p_mat, target.p_mat, ct.c_float(cutoff))
    else:
        raise ValueError("Value must be of type CUDAMatrix.")

    if err_code:
        raise generate_exception(err_code)

    return target


def pow(mat, p, target=None):
    """
    If p is a scalar, compute the p-th power of each element of the matrix mat,
    otherwise raise each element of the matrix mat to the power given by the
    corresponding element of the matrix p.
    """

    if not target:
        target = mat

    if isinstance(p, CUDAMatrix):
        err_code = _cudamat.apply_pow_matrix(mat.p_mat, p.p_mat, target.p_mat)
    elif isinstance(p, (int, float)):
        err_code = _cudamat.apply_pow(mat.p_mat, ct.c_float(p), target.p_mat)
    else:
        raise ValueError("Value must be of type CUDAMatrix, int, or float.")

    if err_code:
        raise generate_exception(err_code)

    return target


def cuda_sync_threads():
    """Wait until all cuda threads are finished."""

    _cudamat.cuda_sync_threads()


def reformat(array):
    """Return array as a float32 array in FORTRAN order."""

    if array.dtype != np.float32 or not np.isfortran(array):
        return np.array(array, dtype=np.float32, order='F')
    else:
        return array


def cuda_set_device(dev_id):
    """Select the CUDA device with the given ID."""

    err_code = _cudamat.cuda_set_device(ct.c_int(dev_id))
    if err_code:
        raise generate_exception(err_code)


def cublas_init():
    """Initialize Cublas."""

    _cudamat.cublas_init()
    CUDAMatrix.ones = \
        CUDAMatrix(np.ones((MAX_ONES, 1), dtype=np.float32, order="F"))

init = cublas_init


def cublas_shutdown():
    """Shut down Cublas."""

    CUDAMatrix.ones.free_device_memory()
    CUDAMatrix.ones = 0
    _cudamat.cublas_shutdown()

shutdown = cublas_shutdown



import numpy as np


def logistic(x):
    return 1./(1. + np.exp(-x))

def tanh(x):
    return np.tanh(x)

def softmax(x):
    tmp = x.T - x.max(1) # for numerical stability
    tmp = np.exp(tmp)
    return (tmp/(tmp.sum(0) + 0.01)).T

def inv_cubic(x):
    y = 0.*x
    y_now = y + 1.
    while abs(y_now - y).max() > 1e-6:
        y_now = y
        y = (2./3.*y_now**3 + x)/(y_now**2 + 1)
    return y



import mang.cudamat as cm
from .node import Node


class TanhNode(Node):
    "Node with hyperbolic tangent activation function."""

    _name = "tanh"

    def up(self):
        self._add_b()
        cm.tanh(self.y)

    def down(self):
        self.dy.apply_tanh_deriv(self.y)



from .node import Node
from .softmax_node import SoftmaxNode
from .logistic_node import LogisticNode
from .tanh_node import TanhNode
from .relu_node import ReLUNode
from .response_normalization_node import ResponseNormalizationNode


_NODE_TYPE_LIST = [
    Node,
    TanhNode,
    LogisticNode,
    SoftmaxNode,
    ReLUNode,
    ResponseNormalizationNode
    ]
NODE_TABLE = {}

for node_type in _NODE_TYPE_LIST:
    NODE_TABLE[node_type._name] = node_type



import mang.cudamat as cm
from .node import Node
from . import functions as F


class SoftmaxNode(Node):
    """Node with softmax activation function."""

    _name = "softmax"

    def __init__(self, shape, option):
        Node.__init__(self, shape, option)
        self.y_t = None

    def init_training(self, option):
        Node.init_training(self, option)
        self.y_t = cm.empty((self.size, option["batch_size"]))

    def finish_training(self):
        Node.finish_training(self)
        self.y_t.free_device_memory()
        self.y_t = None

    def up(self):
        if self.shared:
            raise NotImplementedError

        self._add_b()
        if self.y_t is None:
            self.y_t = cm.empty((self.y.shape[1], self.y.shape[0]))
        self.y.transpose(self.y_t)
        cm.softmax(self.y_t)
        self.y_t.transpose(self.y)

    def down(self):
        pass



import numpy as np


def sse(predicted, target, **option):
    return ((predicted - target) ** 2).sum()


def mse(predicted, target, **option):
    return ((predicted - target) ** 2).mean()


def rmse(predicted, target, **option):
    return np.sqrt(((predicted - target) ** 2).mean(1)).mean()


def accuracy(predicted, target, **option):
    label1 = np.argmax(predicted, 1)
    label2 = np.argmax(target, 1)
    return (label1 == label2).mean()


def hamming(predicted, target, **option):
    return (predicted != target).mean()


def one_error(predicted, arget, **option):
    pass


def average_precision(predicted, target, **option):
    pass


def precision_at_k(k):
    return lambda predicted, target: precision(predicted, target, k)


def recall_at_k(k):
    return lambda predicted, target: recall(predicted, target, k)


def precision(predicted, target, k=5):
    N = predicted.shape[0]
    target_list = [set(x.nonzero()[0]) for x in target]
    predicted_list = [set(x.argsort()[:k]) for x in -predicted]
    divisor = [max(min(k, len(x)), 1) for x in target_list]
    n_correct = [len(target_list[i] & predicted_list[i]) for i in xrange(N)]
    return np.mean([1. * n_correct[i] / divisor[i] for i in xrange(N)])


def recall(predicted, target, k=5):
    N = predicted.shape[0]
    target_list = [set(x.nonzero()[0]) for x in target]
    predicted_list = [set(x.argsort()[:k]) for x in -predicted]
    divisor = [max(len(x), 1) for x in target_list]
    n_correct = [len(target_list[i] & predicted_list[i]) for i in xrange(N)]
    return np.mean([1. * n_correct[i] / divisor[i] for i in xrange(N)])


def f1_score(predicted, target, k=5):
    N = predicted.shape[0]
    target_list = [set(x.nonzero()[0]) for x in target]
    predicted_list = [set(x.argsort()[:k]) for x in -predicted]
    n_correct = [len(target_list[i] & predicted_list[i]) for i in xrange(N)]

    divisor = [max(min(k, len(x)), 1) for x in target_list]
    p_list = np.array([1. * n_correct[i] / divisor[i] for i in xrange(N)])

    divisor = [max(len(x), 1) for x in target_list]
    r_list = np.array([1. * n_correct[i] / divisor[i] for i in xrange(N)])

    divisor = [max(1e-6, x) for x in p_list + r_list]

    return (2 * p_list * r_list / divisor).mean()


MEASURE_TABLE = {
    "sse": sse,
    "mse": mse,
    "rmse": rmse,
    "accuracy": accuracy,
    "hamming": hamming,
    "one_error": one_error,
    "precision": precision,
    "recall": recall,
    "f1_score": f1_score,
    }



import pdb
import platform
import warnings
import ctypes as ct
import numpy as np
from cudamat import generate_exception

if platform.system() == 'Windows':
    _cudalearn = ct.cdll.LoadLibrary('libcudalearn.dll')
else:
    _cudalearn = ct.cdll.LoadLibrary('libcudalearn.so')

_cudalearn.mult_by_sigmoid_deriv.restype = ct.c_int

def mult_by_sigmoid_deriv(target, acts):
    """
    target = target * acts * (1 - acts)

    Useful for doing backprop in neural networks with logistic units.
    """

    err_code = _cudalearn.mult_by_sigmoid_deriv(target.p_mat, acts.p_mat)
    if err_code:
        raise generate_exception(err_code)



import numpy as np

import mang.cudamat as cm
from mang.cudamat import cudamat_conv as cm_conv
from mang.edge.edge import Edge


class DeconvolutionalEdge(Edge):

    _name = "deconv"

    def __init__(self, nodes, conn, option):
        self.conn = conn
        (node1, node2) = (nodes[conn[0]], nodes[conn[1]])
        assert node1.shape[0] == node1.shape[1]
        assert node2.shape[0] == node2.shape[1]

        self.dim1 = node1.size
        self.dim2 = node2.size

        # user-defined properties
        self.filter_size = option["filter_size"]
        self.stride = option["stride"] if "stride" in option else 1
        if "padding" in option:
            self.padding = option["padding"]
        else:
            self.padding = int(self.filter_size / 2)

        # derived property
        self.n_locs = (node2.shape[0] + 2 * self.padding - self.filter_size) /\
            self.stride + 1
        self.n_channel = 1 if len(node2.shape) == 2 else node2.shape[-1]
        self.n_filter = 1 if len(node1.shape) == 2 else node1.shape[-1]
        self.shape = \
            (self.n_filter, (self.filter_size ** 2) * self.n_channel)

        if "W" in option:
            self.W = np.array(option["W"], dtype=np.float32, order="F")
        else:
            self.W = np.zeros(self.shape)

        self.on_gpu = False
        self.used_gpu_memory = 0

    def init_training(self, batch_size):
        Edge.init_training(self, batch_size)
        self.W_tmp = cm.empty(self.W.shape)
        self.used_gpu_memory += 4 * self.W.shape[0] * self.W.shape[1]
        self.dy = cm.empty((batch_size, self.dim1))
        self.used_gpu_memory += 4 * batch_size * self.dim1

    def finish_training(self):
        Edge.finish_training(self)
        self.W_tmp.free_device_memory()
        self.used_gpu_memory -= 4 * self.W.shape[0] * self.W.shape[1]
        self.dy.free_device_memory()
        self.used_gpu_memory -= 4 * self.dy.shape[0] * self.dy.shape[1]
        del self.W_tmp

    def up(self, nodes):
        (node1, node2) = (nodes[self.conn[0]], nodes[self.conn[1]])
        cm_conv.convDown(node1.y, self.W, node2.y, self.n_locs, self.padding,
                         self.stride, self.filter_size, node2.shape[0],
                         self.n_channel)

    def down(self, nodes):
        (node1, node2) = (nodes[self.conn[0]], nodes[self.conn[1]])
        cm_conv.convUp(node2.dy, self.W, self.dy, self.n_locs, self.padding,
                       self.stride, self.n_channel)
        node1.dy.add(self.dy)

    def gradient(self, nodes, gW):
        (node1, node2) = (nodes[self.conn[0]], nodes[self.conn[1]])
        cm_conv.convOutp(
            node2.dy, node1.y, self.W_tmp, self.n_locs, self.padding,
            self.filter_size, self.stride, self.n_channel)
        # don't divide gradient by # of samples because it is already averaged
        # over a lot of convolution locations
        self.W_tmp.divide(node2.y.shape[0])
        gW.add(self.W_tmp)

    def to_dict(self):
        """Convert self to a dict."""

        result = Edge.to_dict(self)
        result["filter_size"] = self.filter_size
        result["padding"] = self.padding
        result["stride"] = self.stride
        return result



from .node import Node


class LogisticNode(Node):
    """Node with logistic activation function."""

    _name = "logistic"

    def up(self):
        self._add_b()
        self.y.apply_sigmoid()

    def down():
        self.dy.apply_logistic_deriv(self.y)



import mang.cudamat as cm
from mang.cudamat import cudamat_conv as cm_conv

from .node import Node


class ResponseNormalizationNode(Node):
    """Node that performs response normalization."""

    _name = "rnorm"

    def __init__(self, shape, option):
        option["use_bias"] = False
        Node.__init__(self, shape, option)
        self.norm_size = option["norm_size"]
        self.add_scale = option["add_scale"]
        self.pow_scale = option["pow_scale"]

        self.x = None
        self.cov = None
        self.tmp = None

    def _make_tmp(self):
        self.x = cm.empty(self.y.shape)
        self.cov = cm.empty(self.y.shape)
        self.tmp = cm.empty(self.y.shape)
        self.used_gpu_memory += 12 * self.x.shape[0] * self.x.shape[1]

    def _free_tmp(self):
        self.x.free_device_memory()
        self.cov.free_device_memory()
        self.tmp.free_device_memory()
        self.used_gpu_memory -= 12 * self.x.shape[0] * self.x.shape[1]
        self.x = None
        self.cov = None
        self.tmp = None

    def init_training(self, option):
        Node.init_training(self, option)
        self._make_tmp()

    def finish_training(self):
        Node.finish_training(self)
        self._free_tmp()

    def up(self):
        if self.x is None:
            self._make_tmp()
        self.x.assign(self.y)
        cm_conv.ResponseNorm(
            self.y, self.cov, self.y, self.shape[-1], self.norm_size,
            self.add_scale, self.pow_scale)

    def down(self):
        cm_conv.ResponseNormUndo(self.dy, self.cov, self.y, self.x,
                                 self.tmp, self.shape[-1],
                                 self.norm_size, 1., 1.)
        self.dy.assign(self.tmp)

    def to_dict(self):
        """Convert self into a dictionary."""

        result = Node.to_dict(self)
        result["add_scale"] = self.add_scale
        result["pow_scale"] = self.pow_scale
        result["norm_size"] = self.norm_size

        return result

    @staticmethod
    def from_dict(data):
        """Create a node object from a dictionary."""

        return ResponseNormalizationNode(data["shape"], data)



import pdb
import numpy as np
import nose
import cudamat as cm
import learn as cl

def setup():
    cm.cublas_init()

def teardown():
    cm.cublas_shutdown()

def test_mult_by_sigmoid_deriv():
    m = 256
    n = 128
    c_targets = np.array(np.random.randn(m, n)*10, dtype=np.float32, order='F')
    c_acts = np.array(np.random.rand(m, n), dtype=np.float32, order='F')

    g_targets = cm.CUDAMatrix(c_targets)
    g_acts = cm.CUDAMatrix(c_acts)

    c_targets = c_targets * c_acts * (1. - c_acts)
    cl.mult_by_sigmoid_deriv(g_targets, g_acts)

    assert np.max(np.abs(c_acts - g_acts.asarray())) < 10**-2, "Error in cudamat.learn.mult_by_sigmoid_deriv exceeded threshold"

if __name__ == '__main__':
    nose.run()



import numpy as np

def center(X, mu=None):
    if mu == None:
        mu = X.mean(0)
        return (X - mu, mu)
    else:
        return X - mu

def un_center(X_c, mu):
    return X_c + mu

def whiten(X, stat=None):
    if stat == None:
        N = X.shape[0]
        (X_c, mu) = center(X)
        # tmp = X_c + 0.1*np.random.randn(*X.shape)
        std = np.sqrt((X_c*X_c).sum(0)/N)
        std += 0.1*(std < 0.1)
        X_c = (X_c/std)
        return (X_c, (mu, std))
    else:
        (mu, std) = stat
        X_c = center(X, mu)
        X_c = (X_c/std)
        return X_c

def un_whiten(X_c, stat):
    (mu, std) = stat
    return un_center(X_c*std, mu)

def pca_whiten(X, stat=None, ratio=0.99, dim=None):
    if stat == None:
        (X_c, mu) = center(X)
        (val, vec) = np.linalg.eigh(np.cov(X_c.T))
        idx = np.argsort(val)
        D = len(idx)
        eigval_sum = val.sum()
        eigval_accum = 0
        idx_cutoff = 0
        for i in idx:
            eigval_accum += val[i]
            if dim == None and eigval_accum >= (1 - ratio)*eigval_sum:
                idx_cutoff = i
                break
            elif dim != None and i == D - dim:
                idx_cutoff = i
                break
        idx = idx[idx_cutoff:]
        print len(idx)

        d = val[idx]
        P = vec[:, idx]
        '''
        (d, P) = np.linalg.eigh(np.dot(X_c.T, X_c))
        d += regularizer
        '''
        d = np.sqrt(d)
        d_inv = 1./d
        N = X.shape[0]
        W = P*d_inv
        Winv = (P*d).T
        X_c = np.dot(X_c, W)

        return (X_c, (mu, W, Winv))
    else:
        (mu, W, Winv) = stat
        (X_c, mu) = center(X)
        X_c = np.dot(X_c, W)
        return X_c

def un_pca_whiten(X_c, stat):
    (mu, W, Winv) = stat
    X = un_center(np.dot(X_c, Winv), mu)
    return X

def zca_whiten(X, stat=None, regularizer=0.0001):
    if stat == None:
        N = X.shape[0]
        X_c = (X.T - X.mean(1)).T
        (X_c, mu) = center(X_c)
        (d, P) = np.linalg.eigh(np.dot(X_c.T, X_c)/(N - 1.))
        d += regularizer
        d_inv = 1./np.sqrt(d)
        W = np.dot(P*d_inv, P.T)
        X_c = np.dot(X_c, W)
        X_c = np.clip(X_c, -4, 4)
        return (X_c, (mu, W))
    else:
        (mu, W) = stat
        X_c = (X.T - X.mean(1)).T
        X_c = center(X_c, mu)
        X_c = np.dot(X_c, W)
        X_c = np.clip(X_c, -4, 4)
        return X_c

def un_zca_whiten(X_c, stat):
    raise NotImplementedError



import time

import numpy as np
import mang.cudamat as cm

from mang.measure import MEASURE_TABLE
from mang.cost import D_COST_TABLE
from mang.noise import NOISE_TABLE
from mang import graph
from mang import node as mnode
from mang import edge as medge
from mang import util


_DEFAULT_OPTION = {
    'reset': True,
    'n_epoch': 300,
    'eps_decay': 0.998,
    'batch_size': 128,
    'callback': None,
    'edge_param': {},
    'node_param': {},
    'verbosity': 0,
    }

_DEFAULT_NODE_PARAM = {
    'eps': 1e-2,
    'init_b': 0,
    'momentum_i': 0.5,
    'momentum_f': 0.99,
    'dropout': 0.,
    'cost': None,
    'cost_param': None,
    'noise': None,
    'noise_param': None,
    'sparsity': 0.,
    'data_from': None,
    }

_DEFAULT_EDGE_PARAM = {
    'eps': 1e-2,
    'init_w': 1e-2,
    'w_norm': None,
    'momentum_i': 0.5,
    'momentum_f': 0.99,
    'decay': 0.,
    }

_EPS = 1e-6


class FeedForwardNetwork(object):
    """Feed-forward network trained using back-propagation."""

    def __init__(self, nodes, edges):
        self.nodes = {}
        for name in nodes:
            spec = nodes[name]
            node_type = mnode.NODE_TABLE[spec['type']]
            self.nodes[name] = node_type(spec['shape'], spec)

        self.edges = {}
        self.ref_count = {}
        for conn in edges:
            spec = edges[conn]
            if spec['type'] != 'ref':
                edge_type = medge.EDGE_TABLE[spec['type']]
                self.edges[conn] = edge_type(self.nodes, conn, spec)
                self.ref_count[conn] = 1.

        # edges that shares weights of other edges
        ref_edges = [x for x in edges if edges[x]['type'] == 'ref']
        self.ref_edges = {}
        for conn in ref_edges:
            spec = edges[conn]
            self.ref_edges[conn] = spec['original']
            self.ref_count[spec['original']] += 1
            edge_type = medge.EDGE_TABLE['ref']
            edge_original = self.edges[spec['original']]
            self.edges[conn] = edge_type(conn, edge_original, spec)

        self.real_edges = set(self.edges.keys()) - set(self.ref_edges.keys())

        (self.inputs, self.outputs) = graph.find_boundary(self.edges.keys())
        self.boundary = self.inputs | self.outputs

        # ff_edges: outgoing edges, bp_edges: incoming edges
        self.ff_edges = dict(
            [(name, [x[1] for x in self.edges if x[0] == name])
                for name in self.nodes])
        self.bp_edges = dict(
            [(name, [x[0] for x in self.edges if x[1] == name])
                for name in self.nodes])

        self.ff_order = graph.find_order(self.edges.keys())
        self.bp_order = [x for x in self.ff_order if x not in self.inputs]
        self.bp_order.reverse()

        self.is_training = False
        self.train_option = None
        self.edge_param = None
        self.node_param = None

    def _init_training(self, option):
        """Initialize training parameters."""

        # load default options
        _option = dict(_DEFAULT_OPTION)
        for key in option:
            _option[key] = option[key]
        option = _option

        used_gpu_memory = 0
        self.edge_param = dict(option['edge_param'])

        for conn in self.real_edges:
            param = dict(_DEFAULT_EDGE_PARAM)
            if conn in self.edge_param:
                for key in self.edge_param[conn]:
                    param[key] = self.edge_param[conn][key]
            self.edge_param[conn] = dict(param)
            self.edges[conn].W = param['init_w'] * \
                np.random.randn(*self.edges[conn].W.shape)
            self.edges[conn].init_training(option['batch_size'])
            used_gpu_memory += self.edges[conn].used_gpu_memory

        self.node_param = dict(option['node_param'])
        for name in self.nodes:
            param = dict(_DEFAULT_NODE_PARAM)
            if name in self.node_param:
                for key in self.node_param[name]:
                    param[key] = self.node_param[name][key]
            self.node_param[name] = dict(param)
            if self.nodes[name].use_bias:
                self.nodes[name].b = \
                    param['init_b'] * np.ones(self.nodes[name].b.shape)
            self.nodes[name].init_training(option)
            used_gpu_memory += self.nodes[name].used_gpu_memory

        # temporary variables used for training
        # activations, and their gradients
        self._g = {'data': {}, 'db': {}, 'gb': {}, 'dW': {}, 'gW': {}, }

        # allocate spaces for data and biases
        cm.cublas_init()
        cm.CUDAMatrix.init_random()

        batch_size = option['batch_size']
        for name in self.boundary:
            data_from = self.node_param[name]['data_from']
            if data_from is None:
                self._g['data'][name] = \
                    cm.empty((batch_size, self.nodes[name].size))
                used_gpu_memory += 4 * batch_size * self.nodes[name].size

        for name in self.nodes:
            if self.nodes[name].use_bias:
                self._g['db'][name] = cm.empty(self.nodes[name].b.shape)
                self._g['gb'][name] = cm.empty(self.nodes[name].b.shape)
                self._g['db'][name].assign(0)
                self._g['gb'][name].assign(0)
                used_gpu_memory += 8 * self.nodes[name].b.shape[1]

        for conn in self.real_edges:
            self._g['dW'][conn] = cm.empty(self.edges[conn].W.shape)
            self._g['gW'][conn] = cm.empty(self.edges[conn].W.shape)
            self._g['dW'][conn].assign(0)
            self._g['gW'][conn].assign(0)
            used_gpu_memory += \
                8 * self.edges[conn].W.shape[0] * self.edges[conn].W.shape[1]

        # allocate spaces for dropout masks (still not sure if it's needed)
        self._g['mask'] = {}
        for name in self.nodes:
            if self.node_param[name]['dropout'] > 0.:
                self._g['mask'][name] = \
                    cm.empty((batch_size, self.nodes[name].size))
                used_gpu_memory += 4 * batch_size * self.nodes[name].size

        self.train_option = dict(option)
        self.is_training = True

        if self.train_option['verbosity'] > 0:
            print "Using %dMB of GPU memory" % (used_gpu_memory >> 20)

    def _finish_training(self):
        """store the trained parameters and remove GPU variables"""

        for conn in self.real_edges:
            self.edges[conn].finish_training()

        # un-link the tied edges.
        for conn in self.ref_edges:
            self.edges[conn] = self.edges[conn].materialize(self.nodes)
        self.ref_edges = {}
        self.real_edges = set(self.edges.keys())

        for name in self.nodes:
            self.nodes[name].finish_training()
            if self.node_param[name]['dropout'] > 0.:
                for name2 in self.ff_edges[name]:
                    conn = (name, name2)
                    self.edges[conn].W *= self.node_param[name]['dropout']

        del self._g, self.train_option, self.node_param, self.edge_param
        self.is_training = False

    def fit(self, data, **option):
        """Train network with given data and training options."""

        self._init_training(option)
        stat = []
        time_train = 0
        for i_epoch in xrange(self.train_option['n_epoch']):
            t_epoch_start = time.time()
            for conn in self.edge_param:
                self.edge_param[conn]['eps'] *= self.train_option['eps_decay']
            for name in self.node_param:
                self.node_param[name]['eps'] *= self.train_option['eps_decay']

            if i_epoch < 500:
                r_epoch = i_epoch / 500.
                for name in self.node_param:
                    momentum_f = self.node_param[name]['momentum_f']
                    momentum_i = self.node_param[name]['momentum_i']
                    self.node_param[name]['momentum'] = \
                        r_epoch * momentum_f + (1. - r_epoch) * momentum_i
                for conn in self.edge_param:
                    momentum_f = self.edge_param[conn]['momentum_f']
                    momentum_i = self.edge_param[conn]['momentum_i']
                    self.edge_param[conn]['momentum'] = \
                        r_epoch * momentum_f + (1. - r_epoch) * momentum_i
            else:
                for name in self.node_param:
                    self.node_param[name]['momentum'] = \
                        self.node_param[name]['momentum_f']
                for conn in self.edge_param:
                    self.edge_param[conn]['momentum'] = \
                        self.edge_param[conn]['momentum_f']

            n_iter = self.fit_epoch(data)
            time_train += time.time() - t_epoch_start
            stat += [{
                'epoch': i_epoch,
                'iter': n_iter,
                'time': time_train,
                'train_cost': None,
                'validation_cost': None,
                'train_measure': None,
                'validation_measure': None,
                'node_param': dict(self.node_param),
                'edge_param': dict(self.edge_param),
                }]

            if self.train_option['callback'] is not None:
                self.train_option['callback'](self, stat)

            # restore temporary GPU variables to prevent potential problems
            for name in self.nodes:
                self.nodes[name].to_gpu(self.train_option['batch_size'])
            for conn in self.real_edges:
                self.edges[conn].to_gpu(self.train_option['batch_size'])

        self._finish_training()

        return stat

    def fit_epoch(self, data):
        """Train network for one epoch (single sweep over training data)."""

        if isinstance(data, list):
            n_iter = 0
            for filename in data:
                data_item = util.load_file(filename)
                n_iter += self.fit_epoch(data_item)
            return n_iter

        n_sample = data[data.keys()[0]].shape[0]
        batch_size = self.train_option['batch_size']

        for name in data:
            data[name] = np.array(data[name], dtype=np.float32, order='F')

        # shuffle data
        rng_state = np.random.get_state()
        for name in data:
            np.random.set_state(rng_state)
            np.random.shuffle(data[name])

        n_batch = int(n_sample / batch_size)
        i_start = 0
        for _ in xrange(n_batch):
            i_end = i_start + batch_size
            for name in data:
                self._g['data'][name].overwrite(data[name][i_start:i_end])
                if name in self.inputs:
                    self.nodes[name].y.overwrite(data[name][i_start:i_end])
            self.fit_step()  # mini-batch training
            i_start = i_end
        self._check_values()

        return n_batch

    def fit_step(self):
        """Perform one step of backpropagation learning."""

        # add noise to training data
        for name in self.inputs:
            if self.node_param[name]['noise'] is not None:
                noise_name = self.node_param[name]['noise']
                noise_param = self.node_param[name]['noise_param']
                noise_func = NOISE_TABLE[noise_name]
                noise_func(self.nodes[name].y, noise_param)

        self._feed_forward()
        self._back_propagate()
        self._update()

        '''
        # renormalize ReLU weights so that their maximum output is 1
        skip_nodes = set(self.boundary)
        for conn in self.ref_edges:
            skip_nodes |= set(conn)
            for conn2 in self.ref_edges:
                skip_nodes |= set(conn2)
        for name in self.bp_order:
            if not isinstance(self.nodes[name], mnode.ReLUNode):
                continue
            if name not in skip_nodes:
                shape_old = self.nodes[name].y.shape
                self.nodes[name].y.reshape((shape_old[0] * shape_old[1], 1))
                y_max = float(self.nodes[name].y.max(0).asarray())
                self.nodes[name].y.reshape(shape_old)
                if y_max <= 1.:
                    continue
                self.nodes[name].b.divide(y_max)
                for name2 in self.bp_edges[name]:
                    self.edges[name2, name].W.divide(y_max)
                for name2 in self.ff_edges[name]:
                    self.edges[name, name2].W.mult(y_max)
        '''

    def _feed_forward(self):
        """
        Calculate activations of all nodes in network,
        using mini-batch stored in GPU as input.
        """

        for name in self.nodes:
            if name not in self.inputs:
                self.nodes[name].y.assign(0)
        for name in self.ff_order:
            if name not in self.inputs:
                self.nodes[name].up()
                # dropout: randomly drop hidden nodes using binary masks
                if self.is_training:
                    dropout = self.node_param[name]['dropout']
                    if dropout > 0.:
                        self._g['mask'][name].assign(1.)
                        self._g['mask'][name].dropout(dropout)
                        self.nodes[name].y.mult(self._g['mask'][name])
            for name2 in self.ff_edges[name]:
                conn = (name, name2)
                self.edges[conn].up(self.nodes)

        '''
        for name in self.nodes:
            data_from = self.node_param[name]['data_from']
            if data_from is not None:
                if data_from in self._g['data']:
                    self._g["data"][name].assign(self._g['data'][data_from])
                else:
                    self._g["data"][name].assign(self.nodes[data_from].y)
        '''

    def _back_propagate(self):
        """
        Back-propagate through network and calculate gradients of
        objective functions with respect to parameters.

        Todo: reduce the number of branches (how?).
        """

        for name in self.nodes:
            self.nodes[name].dy.assign(0)

        for name in self.outputs:
            # assign ground-truth values
            data_from = self.node_param[name]['data_from']
            if data_from is not None:
                if data_from in self._g['data']:
                    truth = self._g['data'][data_from]
                else:
                    truth = self.nodes[data_from].y
            else:
                truth = self._g['data'][name]

            # apply cost functions to output nodes
            cost_name = self.node_param[name]['cost']
            if cost_name is None:
                # zero error for non-data output nodes.
                if self.node_param[name]['sparsity'] > 0:
                    lamb = -self.node_param[name]['sparsity']
                    self.nodes[name].dy.add_mult_sign(self.nodes[name].y, lamb)
                self.nodes[name].down()
            # handle special cases for faster calculation
            elif isinstance(self.nodes[name], mnode.SoftmaxNode) or \
                    (isinstance(self.nodes[name], mnode.LogisticNode) and
                        cost_name == 'cross_entropy'):
                truth.subtract(self.nodes[name].y, self.nodes[name].dy)
            else:
                cost_func = D_COST_TABLE[cost_name]
                cost_func(self.nodes[name].y, truth, self.nodes[name].dy)
                self.nodes[name].down()

        for conn in self.real_edges:
            self._g['gW'][conn].assign(0)
        for name in self.bp_order:
            if name not in self.outputs:
                if self.node_param[name]['sparsity'] > 0:
                    lamb = -self.node_param[name]['sparsity']
                    self.nodes[name].dy.add_mult_sign(self.nodes[name].y, lamb)
                self.nodes[name].down()
            if self.nodes[name].use_bias:
                self.nodes[name].gradient(self._g['gb'][name])
            # dropout: randomly drop hidden nodes using binary masks
            dropout = self.node_param[name]['dropout']
            if dropout > 0. and name not in self.outputs:
                self.nodes[name].dy.mult(self._g['mask'][name])
            for name2 in self.bp_edges[name]:
                conn = (name2, name)
                if conn in self.ref_edges:
                    conn2 = self.ref_edges[conn]
                else:
                    conn2 = conn
                self.edges[conn].gradient(self.nodes, self._g['gW'][conn2])
                self.edges[conn].down(self.nodes)

    def _update(self):
        """
        Update parameters using the gradients calculated in _back_propagate.
        """

        for conn in self.real_edges:
            momentum = self.edge_param[conn]['momentum']
            eps = (1. - momentum) * self.edge_param[conn]['eps']
            decay = self.edge_param[conn]['decay']
            self._g['dW'][conn].mult(momentum)
            self._g['dW'][conn].add_mult(
                self._g['gW'][conn], eps / self.ref_count[conn])
            if decay > 0:
                self._g['dW'][conn].add_mult(self.edges[conn].W, - eps * decay)
            self.edges[conn].W.add(self._g['dW'][conn])

            # limit the squared norm of features
            if self.edge_param[conn]['w_norm'] is not None:
                w_norm_limit = self.edge_param[conn]['w_norm']
                if isinstance(self.edges[conn], medge.ConvolutionalEdge):
                    self.edges[conn].W.norm_limit(w_norm_limit, 1)
                elif isinstance(self.edges[conn], medge.DeconvolutionalEdge):
                    self.edges[conn].W.norm_limit(w_norm_limit, 1)
                else:
                    self.edges[conn].W.norm_limit(w_norm_limit, 0)

        for name in self.nodes:
            if name not in self.inputs and self.nodes[name].use_bias:
                momentum = self.node_param[name]['momentum']
                eps = (1. - momentum) * self.node_param[name]['eps']
                self._g['db'][name].mult(momentum)
                self._g['db'][name].add_mult(self._g['gb'][name], eps)
                self.nodes[name].b.add(self._g['db'][name])

    def _check_values(self):
        """Check if there is any NaN/inf in model parameters."""

        for conn in self.real_edges:
            assert np.isinf(self.edges[conn].W.asarray()).sum() == 0
            assert np.isnan(self.edges[conn].W.asarray()).sum() == 0
        for name in self.nodes:
            if name not in self.inputs and self.nodes[name].use_bias:
                assert np.isinf(self.nodes[name].b.asarray()).sum() == 0
                assert np.isnan(self.nodes[name].b.asarray()).sum() == 0

    def feed_forward(self, data, batch_size=128, nodes=None):
        """Calculate output node activations from input data."""

        for name in data:
            data[name] = np.array(data[name], dtype=np.float32, order='F')
        n_sample = data[data.keys()[0]].shape[0]
        result = {}
        if nodes is None:
            nodes = self.outputs
        for name in nodes:
            result[name] = np.zeros((n_sample, self.nodes[name].size))

        # initialize GPU variables of nodes and edges
        if self.is_training:
            batch_size = self.train_option['batch_size']
        for name in self.nodes:
            self.nodes[name].to_gpu(batch_size)
        for conn in self.real_edges:
            self.edges[conn].to_gpu(batch_size)

        n_batch = int(np.ceil(n_sample / batch_size))
        for i_batch in xrange(n_batch):
            i_start = i_batch * batch_size
            i_end = min(n_sample, i_start + batch_size)
            real_batch_size = i_end - i_start
            for name in self.inputs:
                tmp = np.zeros((batch_size, self.nodes[name].size),
                               dtype=np.float32, order='F')
                tmp[:real_batch_size] = data[name][i_start:i_end]
                self.nodes[name].y.overwrite(tmp)
            self._feed_forward()
            for name in result:
                result[name][i_start:i_end] = self.nodes[name].y.asarray()

        for name in self.nodes:
            self.nodes[name].from_gpu()
        for conn in self.real_edges:
            self.edges[conn].from_gpu()

        return result

    def evaluate(self, data, measures, option={}, n_max=None):
        """Evaluate performance of network using data and measure functions."""

        for key in data:
            data[key] = np.array(data[key], dtype=np.float32, order='F')
            if n_max is not None:
                data[key] = data[key][:n_max]
        predicted = self.feed_forward(data)
        result = {}
        for name in measures:
            measure_func = MEASURE_TABLE[measures[name]]
            result[name] = measure_func(predicted[name], data[name], **option)
        return result

    def save(self, filename):
        """Save current model as a file."""

        result = {
            # save node/edge parameters
            'nodes': dict([(x, self.nodes[x].to_dict()) for x in self.nodes]),
            'edges': dict([(x, self.edges[x].to_dict()) for x in self.edges]),

            # save topology
            'inputs': list(self.inputs),
            'outputs': list(self.outputs),
            'ref_edges': self.ref_edges,
            'ff_edges': self.ff_edges,
            'bp_edges': self.bp_edges,
            'ff_order': self.ff_order,
            'bp_order': self.bp_order,

            # additional information
            'ref_count': self.ref_count,
            }
        util.save_msgpack(filename, result)

    @staticmethod
    def load(filename):
        """Load a network from file and return the network object."""

        net = FeedForwardNetwork({}, {})
        result = util.load_msgpack(filename)

        net.inputs = set(result['inputs'])
        net.outputs = set(result['outputs'])
        net.ref_edges = result['ref_edges']

        net.ff_edges = result['ff_edges']
        net.bp_edges = result['bp_edges']

        net.ff_order = result['ff_order']
        net.bp_order = result['bp_order']

        net.ref_count = result['ref_count']
        net.is_training = False

        net.nodes = {}
        for name in result['nodes']:
            info = result['nodes'][name]
            net.nodes[name] = mnode.NODE_TABLE[info['type']].from_dict(info)

        net.real_edges = \
            set(result['edges'].keys()) - set(net.ref_edges.keys())

        for conn in net.real_edges:
            info = result['edges'][conn]
            net.edges[conn] = \
                medge.EDGE_TABLE[info['type']].from_dict(net.nodes, info)

        for conn in net.ref_edges:
            info = result['edges'][conn]
            original = net.edges[net.ref_edges[conn]]
            net.edges[conn] = \
                medge.EDGE_TABLE['ref'].from_dict(original, info)

        return net



from mang.edge import Edge


class EdgeRef(object):

    _name = "ref"

    def __init__(self, conn, original, option):
        self.conn = conn
        self.original = original
        self.transpose = option["transpose"]

    def init_training(self, option):
        pass

    def finish_training(self):
        pass

    def up(self, nodes):
        (node1, node2) = (nodes[self.conn[0]], nodes[self.conn[1]])
        if self.transpose:
            node2.y.add_dot(node1.y, self.original.W.T)
        else:
            node2.y.add_dot(node1.y, self.original.W)

    def down(self, nodes):
        (node1, node2) = (nodes[self.conn[0]], nodes[self.conn[1]])
        if self.transpose:
            node1.dy.add_dot(node2.dy, self.original.W)
        else:
            node1.dy.add_dot(node2.dy, self.original.W.T)

    def gradient(self, nodes, gW):
        (node1, node2) = (nodes[self.conn[0]], nodes[self.conn[1]])
        if self.transpose:
            gW.add_dot(node2.dy.T, node1.y, 1. / node1.y.shape[0])
        else:
            gW.add_dot(node1.y.T, node2.dy, 1. / node1.y.shape[0])

    def materialize(self, nodes):
        W = self.original.W if self.transpose else self.original.W.T
        return Edge(nodes, self.conn, W)

    def to_dict(self):
        """Convert self to a dict."""

        result = {"transpose": self.transpose, "conn": self.conn, }
        return result

    @classmethod
    def from_dict(cls, original, data):
        """Create an edge reference object from data."""

        return cls(data["conn"], original, data)



import Image
import numpy as np


def filter_image(mat, shape, order="F"):
    n = mat.shape[0]
    w = int(np.sqrt(n))
    h = int(np.ceil(1. * n / w))
    if len(shape) == 2:
        arr = np.zeros((w * (shape[0] + 1) - 1, h * (shape[1] + 1) - 1))
    elif len(shape) == 3:
        if shape[2] == 1:
            arr = np.zeros((w * (shape[0] + 1) - 1, h * (shape[1] + 1) - 1))
        elif shape[2] == 3:  # RGB image
            arr = np.zeros((w * (shape[0] + 1) - 1, h * (shape[1] + 1) - 1, 3))
        else:
            w_p = int(np.sqrt(shape[2]))
            h_p = int(np.ceil(1. * shape[2] / w_p))

            arr = np.zeros((w * ((shape[0] + 1) * w_p + 2) - 3,
                            h * ((shape[1] + 1) * h_p + 2) - 3))
    else:
        raise NotImplementedError
    i_img = 0
    for i_y in xrange(h):
        for i_x in xrange(w):
            patch = mat[i_img].reshape(shape, order=order)
            patch -= patch.min()
            patch /= patch.max()
            i1 = i_x * (shape[0] + 1)
            i2 = i1 + shape[0]
            if len(shape) == 2 or shape[2] == 3:
                j1 = i_y * (shape[1] + 1)
                j2 = j1 + shape[1]
                arr[i1:i2, j1:j2] = patch
            elif shape[2] == 1:
                j1 = i_y * (shape[1] + 1)
                j2 = j1 + shape[1]
                arr[i1:i2, j1:j2] = patch[:, :, 0].squeeze()
            else:
                i_ch = 0
                for i_y_p in xrange(h_p):
                    for i_x_p in xrange(w_p):
                        i1 = i_x * (w_p * (shape[0] + 1) + 2) + \
                            i_x_p * (shape[0] + 1)
                        i2 = i1 + shape[0]
                        j1 = i_y * (h_p * (shape[1] + 1) + 2) + \
                            i_y_p * (shape[1] + 1)
                        j2 = j1 + shape[1]
                        arr[i1:i2, j1:j2] = patch[:, :, i_ch].squeeze()
                        i_ch += 1
                        if i_ch >= shape[2]:
                            break
            i_img += 1
            if i_img >= n:
                break
    if len(arr.shape) == 2:
        arr = arr.T
    else:
        arr = np.transpose(arr, (1, 0, 2))
    image = Image.fromarray(np.uint8(255. * arr))
    return image



#!/usr/bin/python
#coding=utf-8

"""
mang.noise
~~~~~~~~~~

Stochastic noises to be added to inputs of neural networks.

.. moduleauthor:: Yoonseop Kang <e0engoon@gmail.com>

"""
import cudamat as cm


def gaussian(x, option):
    """Add gaussian noise with a fixed variance and return the result."""

    x.sample_gaussian(option)


def salt(x, option):
    """Randomly replace elements of inputs by 1."""

    x.dropout(option, 1.)


def pepper(x, option):
    """Randomly replace elements of inputs by 0."""

    x.dropout(option, 0.)


NOISE_TABLE = {
    "gaussian": gaussian,
    "salt": salt,
    "pepper": pepper,
    }



import cPickle as pickle
import gzip
import time

import numpy as np
import scipy.io as sio
import msgpack
import msgpack_numpy as m
m.patch()


_TIME = None


def tic():
    global _TIME
    _TIME = time.time()


def toc():
    return time.time() - _TIME


def loadDefault(given, default):
    result = dict(default)
    for key in given:
        result[key] = given[key]
    return result


def save_pickle(filename, data):
    with open(filename, 'wb') as fp:
        pickle.dump(data, fp)
        fp.close()


def load_pickle(filename):
    with open(filename, 'rb') as fp:
        data = pickle.load(fp)
    return data


# use it to save models
def save_pkl_gz(filename, data):
    with gzip.open(filename, 'wb') as fp:
        pickle.dump(data, fp)
        fp.close()


def load_pkl_gz(filename):
    with gzip.open(filename, 'rb') as fp:
        data = pickle.load(fp)
    return data


# use it to save data
def save_npz(filename, data):
    if type(data) == dict:
        np.savez(filename, **data)
    else:
        np.save(filename, data)


def load_npz(filename):
    data = np.load(filename)
    return data


def save_msgpack(filename, data):
    with open(filename, "wb") as fp:
        fp.write(msgpack.packb(data))


def load_msgpack(filename):
    with open(filename, "rb") as fp:
        data = msgpack.unpackb(fp.read())
        return data


def load_file(filename):
    ext_table = {
        ".pkl": load_pickle,
        ".pkl.gz": load_pkl_gz,
        ".npy": np.load,
        ".npz": np.load,
        ".msgpack": load_msgpack,
        ".mat": sio.loadmat,
        }

    for ext in ext_table:
        if filename.endswith(ext):
            return ext_table[ext](filename)

    raise ValueError("Unknown file format")
